@@ -19,7 +19,7 @@ double imp_match_out_sc( int i1, int j1 )
 	return( impmtx[i1][j1] );
 }
 
-#if 1
+#if 0
 static void imp_match_out_vead_gapmap( double *imp, int i1, int lgth2, int *gapmap2 )
 {
 #if FASTMATCHCALC
@@ -75,25 +75,25 @@ void imp_rna( int nseq1, int nseq2, char **seq1, char **seq2, double *eff1, doub
 	foldrna( nseq1, nseq2, seq1, seq2, eff1, eff2, grouprna1, grouprna2, impmtx, gapmap1, gapmap2, pair );
 }
 
-void imp_match_init_strict( double *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, double *eff1_kozo, double *eff2_kozo, LocalHom ***localhom, int forscore )
+void imp_match_init_strict( double *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, double *eff1_kozo, double *eff2_kozo, LocalHom ***localhom, char *swaplist, int forscore, int *orinum1, int *orinum2 )
 {
-	int i, j, k1, k2, tmpint, start1, start2, end1, end2;
-	double effij;
-	double effij_kozo;
-	double effijx;
-	char *pt, *pt1, *pt2;
-	static TLS char *nocount1 = NULL;
-	static TLS char *nocount2 = NULL;
-	LocalHom *tmpptr;
+//	int i, j, k1, k2, tmpint, start1, start2, end1, end2;
+//	double effij;
+//	double effij_kozo;
+//	double effijx;
+//	char *pt, *pt1, *pt2;
+//	static TLS char *nocount1 = NULL;
+//	static TLS char *nocount2 = NULL;
+//	LocalHom *tmpptr;
 
 	if( seq1 == NULL )
 	{
 		if( impmtx ) FreeFloatMtx( impmtx );
 		impmtx = NULL;
-		if( nocount1 ) free( nocount1 );
-		nocount1 = NULL;
-		if( nocount2 ) free( nocount2 );
-		nocount2 = NULL;
+//		if( nocount1 ) free( nocount1 );
+//		nocount1 = NULL;
+//		if( nocount2 ) free( nocount2 );
+//		nocount2 = NULL;
 		
 		return;
 	}
@@ -101,363 +101,17 @@ void imp_match_init_strict( double *imp, int clus1, int clus2, int lgth1, int lg
 	if( impalloclen < lgth1 + 2 || impalloclen < lgth2 + 2 )
 	{
 		if( impmtx ) FreeFloatMtx( impmtx );
-		if( nocount1 ) free( nocount1 );
-		if( nocount2 ) free( nocount2 );
+//		if( nocount1 ) free( nocount1 );
+//		if( nocount2 ) free( nocount2 );
 		impalloclen = MAX( lgth1, lgth2 ) + 2;
 		impmtx = AllocateFloatMtx( impalloclen, impalloclen );
-		nocount1 = AllocateCharVec( impalloclen );
-		nocount2 = AllocateCharVec( impalloclen );
+//		nocount1 = AllocateCharVec( impalloclen );
+//		nocount2 = AllocateCharVec( impalloclen );
 	}
 
-	for( i=0; i<lgth1; i++ )
-	{
-		for( j=0; j<clus1; j++ )
-			if( seq1[j][i] == '-' ) break;
-		if( j != clus1 ) nocount1[i] = 1; 
-		else			 nocount1[i] = 0;
-	}
-	for( i=0; i<lgth2; i++ )
-	{
-		for( j=0; j<clus2; j++ )
-			if( seq2[j][i] == '-' ) break;
-		if( j != clus2 ) nocount2[i] = 1;
-		else			 nocount2[i] = 0;
-	}
-
-#if 0
-fprintf( stderr, "nocount2 =\n" );
-for( i = 0; i<impalloclen; i++ )
-{
-	fprintf( stderr, "nocount2[%d] = %d (%c)\n", i, nocount2[i], seq2[0][i] );
-}
-#endif
-
-
-
-#if 0
-	fprintf( stderr, "eff1 in _init_strict = \n" );
-	for( i=0; i<clus1; i++ )
-		fprintf( stderr, "eff1[] = %f\n", eff1[i] );
-	for( i=0; i<clus2; i++ )
-		fprintf( stderr, "eff2[] = %f\n", eff2[i] );
-#endif
-
-	for( i=0; i<lgth1; i++ ) for( j=0; j<lgth2; j++ )
-		impmtx[i][j] = 0.0;
-	effijx =  fastathreshold;
-	for( i=0; i<clus1; i++ )
-	{
-		for( j=0; j<clus2; j++ )
-		{
-			effij = (double)( eff1[i] * eff2[j] * effijx );
-			effij_kozo = (double)( eff1_kozo[i] * eff2_kozo[j] * effijx );
-			tmpptr = localhom[i][j];
-			while( tmpptr )
-			{
-//				fprintf( stderr, "start1 = %d\n", tmpptr->start1 );
-//				fprintf( stderr, "end1   = %d\n", tmpptr->end1   );
-//				fprintf( stderr, "i = %d, seq1 = \n%s\n", i, seq1[i] );
-//				fprintf( stderr, "j = %d, seq2 = \n%s\n", j, seq2[j] );
-				pt = seq1[i];
-				tmpint = -1;
-				while( *pt != 0 )
-				{
-					if( *pt++ != '-' ) tmpint++;
-					if( tmpint == tmpptr->start1 ) break;
-				}
-				start1 = pt - seq1[i] - 1;
-	
-				if( tmpptr->start1 == tmpptr->end1 ) end1 = start1;
-				else
-				{
-#if MACHIGAI
-					while( *pt != 0 )
-					{
-//						fprintf( stderr, "tmpint = %d, end1 = %d pos = %d\n", tmpint, tmpptr->end1, pt-seq1[i] );
-						if( tmpint == tmpptr->end1 ) break;
-						if( *pt++ != '-' ) tmpint++;
-					}
-					end1 = pt - seq1[i] - 0;
-#else
-					while( *pt != 0 )
-					{
-//						fprintf( stderr, "tmpint = %d, end1 = %d pos = %d\n", tmpint, tmpptr->end1, pt-seq1[i] );
-						if( *pt++ != '-' ) tmpint++;
-						if( tmpint == tmpptr->end1 ) break;
-					}
-					end1 = pt - seq1[i] - 1;
-#endif
-				}
-	
-				pt = seq2[j];
-				tmpint = -1;
-				while( *pt != 0 )
-				{
-					if( *pt++ != '-' ) tmpint++;
-					if( tmpint == tmpptr->start2 ) break;
-				}
-				start2 = pt - seq2[j] - 1;
-				if( tmpptr->start2 == tmpptr->end2 ) end2 = start2;
-				else
-				{
-#if MACHIGAI
-					while( *pt != 0 )
-					{
-						if( tmpint == tmpptr->end2 ) break;
-						if( *pt++ != '-' ) tmpint++;
-					}
-					end2 = pt - seq2[j] - 0;
-#else
-					while( *pt != 0 )
-					{
-						if( *pt++ != '-' ) tmpint++;
-						if( tmpint == tmpptr->end2 ) break;
-					}
-					end2 = pt - seq2[j] - 1;
-#endif
-				}
-//				fprintf( stderr, "start1 = %d (%c), end1 = %d (%c), start2 = %d (%c), end2 = %d (%c)\n", start1, seq1[i][start1], end1, seq1[i][end1], start2, seq2[j][start2], end2, seq2[j][end2] );
-//				fprintf( stderr, "step 0\n" );
-				if( end1 - start1 != end2 - start2 )
-				{
-//					fprintf( stderr, "CHUUI!!, start1 = %d, end1 = %d, start2 = %d, end2 = %d\n", start1, end1, start2, end2 );
-				}
-
-#if 1
-				k1 = start1; k2 = start2;
-				pt1 = seq1[i] + k1;
-				pt2 = seq2[j] + k2;
-				while( *pt1 && *pt2 )
-				{
-					if( *pt1 != '-' && *pt2 != '-' )
-					{
-// ŤߤŤˤʤ褦դƲ
-//						impmtx[k1][k2] += tmpptr->wimportance * fastathreshold;
-//						impmtx[k1][k2] += tmpptr->importance * effij;
-//						impmtx[k1][k2] += tmpptr->fimportance * effij;
-						if( tmpptr->korh == 'k' )
-							impmtx[k1][k2] += tmpptr->fimportance * effij_kozo;
-						else
-							impmtx[k1][k2] += tmpptr->fimportance * effij;
-							
-//						fprintf( stderr, "#### impmtx[k1][k2] = %f, tmpptr->fimportance=%f, effij=%f\n", impmtx[k1][k2], tmpptr->fimportance, effij );
-//						fprintf( stderr, "mark, %d (%c) - %d (%c) \n", k1, *pt1, k2, *pt2 );
-//						fprintf( stderr, "%d (%c) - %d (%c)  - %f\n", k1, *pt1, k2, *pt2, tmpptr->fimportance * effij );
-						k1++; k2++;
-						pt1++; pt2++;
-					}
-					else if( *pt1 != '-' && *pt2 == '-' )
-					{
-//						fprintf( stderr, "skip, %d (%c) - %d (%c) \n", k1, *pt1, k2, *pt2 );
-						k2++; pt2++;
-					}
-					else if( *pt1 == '-' && *pt2 != '-' )
-					{
-//						fprintf( stderr, "skip, %d (%c) - %d (%c) \n", k1, *pt1, k2, *pt2 );
-						k1++; pt1++;
-					}
-					else if( *pt1 == '-' && *pt2 == '-' )
-					{
-//						fprintf( stderr, "skip, %d (%c) - %d (%c) \n", k1, *pt1, k2, *pt2 );
-						k1++; pt1++;
-						k2++; pt2++;
-					}
-					if( k1 > end1 || k2 > end2 ) break;
-				}
-#else
-				while( k1 <= end1 && k2 <= end2 )
-				{
-					fprintf( stderr, "k1,k2=%d,%d - ", k1, k2 );
-					if( !nocount1[k1] && !nocount2[k2] )
-					{
-						impmtx[k1][k2] += tmpptr->wimportance * eff1[i] * eff2[j]  * fastathreshold;
-						fprintf( stderr, "marked\n" );
-					}
-					else
-						fprintf( stderr, "no count\n" );
-					k1++; k2++;
-				}
-#endif
-				tmpptr = tmpptr->next;
-			}
-		}
-	}
-
-#if 0
-	if( clus1 == 1 && clus2 == 1 )
-	{
-		fprintf( stderr, "writing impmtx\n" );
-		fprintf( stderr, "\n" );
-		fprintf( stderr, "seq1[0] =  %s\n", seq1[0] );
-		fprintf( stderr, "seq2[0] =  %s\n", seq2[0] );
-		fprintf( stderr, "impmtx = \n" );
-		for( k2=0; k2<lgth2; k2++ )
-			fprintf( stderr, "%6.3f ", (double)k2 );
-		fprintf( stderr, "\n" );
-		for( k1=0; k1<lgth1; k1++ )
-		{
-			fprintf( stderr, "%d ", k1 );
-			for( k2=0; k2<30; k2++ )
-				fprintf( stderr, "%2.1f ", impmtx[k1][k2] );
-			fprintf( stderr, "\n" );
-		}
-//		exit( 1 );
-	}
-#endif
+	fillimp( impmtx, imp, clus1, clus2, lgth1, lgth2, seq1, seq2, eff1, eff2, eff1_kozo, eff2_kozo, localhom, swaplist, forscore, orinum1, orinum2 );
 }
 
-#if 0
-void imp_match_init( double *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, LocalHom ***localhom )
-{
-	int dif, i, j, k1, k2, tmpint, start1, start2, end1, end2;
-	static TLS int impalloclen = 0;
-	char *pt;
-	int allgap;
-	static TLS char *nocount1 = NULL;
-	static TLS char *nocount2 = NULL;
-
-	if( impalloclen < lgth1 + 2 || impalloclen < lgth2 + 2 )
-	{
-		if( impmtx ) FreeFloatMtx( impmtx );
-		if( nocount1 ) free( nocount1 );
-		if( nocount2 ) free( nocount2 );
-		impalloclen = MAX( lgth1, lgth2 ) + 2;
-		impmtx = AllocateFloatMtx( impalloclen, impalloclen );
-		nocount1 = AllocateCharVec( impalloclen );
-		nocount2 = AllocateCharVec( impalloclen );
-	}
-
-	for( i=0; i<lgth1; i++ )
-	{
-		for( j=0; j<clus1; j++ )
-			if( seq1[j][i] == '-' ) break;
-		if( j != clus1 ) nocount1[i] = 1; 
-		else			 nocount1[i] = 0;
-	}
-	for( i=0; i<lgth2; i++ )
-	{
-		for( j=0; j<clus2; j++ )
-			if( seq2[j][i] == '-' ) break;
-		if( j != clus2 ) nocount2[i] = 1;
-		else			 nocount2[i] = 0;
-	}
-
-#if 0
-fprintf( stderr, "nocount2 =\n" );
-for( i = 0; i<impalloclen; i++ )
-{
-	fprintf( stderr, "nocount2[%d] = %d (%c)\n", i, nocount2[i], seq2[0][i] );
-}
-#endif
-
-	for( i=0; i<lgth1; i++ ) for( j=0; j<lgth2; j++ )
-		impmtx[i][j] = 0;
-	for( i=0; i<clus1; i++ )
-	{
-		fprintf( stderr, "i = %d, seq1 = %s\n", i, seq1[i] );
-		for( j=0; j<clus2; j++ )
-		{
-			fprintf( stderr, "start1 = %d\n", localhom[i][j]->start1 );
-			fprintf( stderr, "end1   = %d\n", localhom[i][j]->end1   );
-			fprintf( stderr, "j = %d, seq2 = %s\n", j, seq2[j] );
-			pt = seq1[i];
-			tmpint = -1;
-			while( *pt != 0 )
-			{
-				if( *pt++ != '-' ) tmpint++;
-				if( tmpint == localhom[i][j]->start1 ) break;
-			}
-			start1 = pt - seq1[i] - 1;
-
-			while( *pt != 0 )
-			{
-//				fprintf( stderr, "tmpint = %d, end1 = %d pos = %d\n", tmpint, localhom[i][j].end1, pt-seq1[i] );
-				if( *pt++ != '-' ) tmpint++;
-				if( tmpint == localhom[i][j]->end1 ) break;
-			}
-			end1 = pt - seq1[i] - 1;
-
-			pt = seq2[j];
-			tmpint = -1;
-			while( *pt != 0 )
-			{
-				if( *pt++ != '-' ) tmpint++;
-				if( tmpint == localhom[i][j]->start2 ) break;
-			}
-			start2 = pt - seq2[j] - 1;
-			while( *pt != 0 )
-			{
-				if( *pt++ != '-' ) tmpint++;
-				if( tmpint == localhom[i][j]->end2 ) break;
-			}
-			end2 = pt - seq2[j] - 1;
-//			fprintf( stderr, "start1 = %d, end1 = %d, start2 = %d, end2 = %d\n", start1, end1, start2, end2 );
-			k1 = start1;
-			k2 = start2;
-			fprintf( stderr, "step 0\n" );
-			while( k1 <= end1 && k2 <= end2 )
-			{
-#if 0
-				if( !nocount1[k1] && !nocount2[k2] )
-					impmtx[k1][k2] += localhom[i][j].wimportance * eff1[i] * eff2[j];
-				k1++; k2++;
-#else
-				if( !nocount1[k1] && !nocount2[k2] )
-					impmtx[k1][k2] += localhom[i][j]->wimportance * eff1[i] * eff2[j];
-				k1++; k2++;
-#endif
-			}
-
-			dif = ( end1 - start1 ) - ( end2 - start2 );
-			fprintf( stderr, "dif = %d\n", dif );
-			if( dif > 0 )
-			{
-				do
-				{
-					fprintf( stderr, "dif = %d\n", dif );
-					k1 = start1;
-					k2 = start2 - dif;
-					while( k1 <= end1 && k2 <= end2 )
-					{
-						if( 0 <= k2 && start2 <= k2 && !nocount1[k1] && !nocount2[k2] )
-							impmtx[k1][k2] = localhom[i][j]->wimportance * eff1[i] * eff2[j];
-						k1++; k2++;
-					}
-				}
-				while( dif-- );
-			}
-			else
-			{
-				do
-				{
-					k1 = start1 + dif;
-					k2 = start2;
-					while( k1 <= end1 )
-					{
-						if( k1 >= 0 && k1 >= start1 && !nocount1[k1] && !nocount2[k2] )
-							impmtx[k1][k2] = localhom[i][j]->wimportance * eff1[i] * eff2[j];
-						k1++; k2++;
-					}
-				}
-				while( dif++ );
-			}
-		}
-	}
-#if 0
-	fprintf( stderr, "impmtx = \n" );
-	for( k2=0; k2<lgth2; k2++ )
-		fprintf( stderr, "%6.3f ", (double)k2 );
-	fprintf( stderr, "\n" );
-	for( k1=0; k1<lgth1; k1++ )
-	{
-		fprintf( stderr, "%d", k1 );
-		for( k2=0; k2<lgth2; k2++ )
-			fprintf( stderr, "%6.3f ", impmtx[k1][k2] );
-		fprintf( stderr, "\n" );
-	}
-#endif
-}
-#endif
 
 static void match_calc_del( int **which, double ***matrices, double *match, int n1, char **seq1, double *eff1, int n2, char **seq2, double *eff2, int i1, int lgth2, int mid, int nmask, int *mask1, int *mask2 ) 
 {
@@ -742,7 +396,8 @@ static void Atracking_localhom( double *impwmpt, double *lasthorizontalw, double
 						char **seq1, char **seq2, 
                         char **mseq1, char **mseq2, 
                         int **ijp, int icyc, int jcyc,
-						int *warpis, int *warpjs, int warpbase )
+						int *warpis, int *warpjs, int warpbase,
+						int *ngap1, int *ngap2, int reuseprofiles )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k, limk;
 	double wm;
@@ -798,6 +453,7 @@ static void Atracking_localhom( double *impwmpt, double *lasthorizontalw, double
 	*gaptable1 = 0;
 	gaptable2 = gt2bk + lgth1+lgth2;
 	*gaptable2 = 0;
+	*ngap1 = *ngap2 = 0;
 
 	iin = lgth1; jin = lgth2;
 	limk = lgth1+lgth2 + 1;
@@ -829,12 +485,15 @@ static void Atracking_localhom( double *impwmpt, double *lasthorizontalw, double
 				*--gaptable1 = 'o';
 				*--gaptable2 = '-';
 				k++;
+				*ngap2 = 1;
 			}
 			l= jin;
 			while( --l >= 0 )
 			{
 				*--gaptable1 = '-';
 				*--gaptable2 = 'o';
+				k++;
+				*ngap1 = 1;
 			}
 			break;
 		}
@@ -846,6 +505,7 @@ static void Atracking_localhom( double *impwmpt, double *lasthorizontalw, double
 				*--gaptable1 = 'o';
 				*--gaptable2 = '-';
 				k++;
+				*ngap2 = 1;
 			}
 			l= jin - jfi;
 			while( --l )
@@ -853,6 +513,7 @@ static void Atracking_localhom( double *impwmpt, double *lasthorizontalw, double
 				*--gaptable1 = '-';
 				*--gaptable2 = 'o';
 				k++;
+				*ngap1 = 1;
 			}
 		}
 		if( iin == lgth1 || jin == lgth2 )
@@ -870,8 +531,11 @@ static void Atracking_localhom( double *impwmpt, double *lasthorizontalw, double
 		iin = ifi; jin = jfi;
 	}
 
-	for( i=0; i<icyc; i++ ) gapireru( mseq1[i], seq1[i], gaptable1 );
-	for( j=0; j<jcyc; j++ ) gapireru( mseq2[j], seq2[j], gaptable2 );
+	if( *ngap1 || !reuseprofiles )
+		for( i=0; i<icyc; i++ ) gapireru( mseq1[i], seq1[i], gaptable1 );
+
+	if( *ngap2 || !reuseprofiles )
+		for( j=0; j<jcyc; j++ ) gapireru( mseq2[j], seq2[j], gaptable2 );
 
 	free( gt1bk );
 	free( gt2bk );
@@ -882,7 +546,8 @@ static double Atracking( double *lasthorizontalw, double *lastverticalw,
                         char **mseq1, char **mseq2, 
                         int **ijp, int icyc, int jcyc,
 						int tailgp,
-						int *warpis, int *warpjs, int warpbase )
+						int *warpis, int *warpjs, int warpbase,
+						int *ngap1, int *ngap2, int reuseprofiles )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k, limk;
 	double wm;
@@ -940,6 +605,7 @@ static double Atracking( double *lasthorizontalw, double *lastverticalw,
 	gaptable2 = gt2bk + lgth1+lgth2;
 	*gaptable2 = 0;
 
+	*ngap1 = *ngap2 = 0;
 	iin = lgth1; jin = lgth2;
 	limk = lgth1+lgth2 + 1;
 	for( k=0; k<limk; k++ ) 
@@ -970,12 +636,15 @@ static double Atracking( double *lasthorizontalw, double *lastverticalw,
 				*--gaptable1 = 'o';
 				*--gaptable2 = '-';
 				k++;
+				*ngap2=1;
 			}
 			l= jin;
 			while( --l >= 0 )
 			{
 				*--gaptable1 = '-';
 				*--gaptable2 = 'o';
+				k++;
+				*ngap1=1;
 			}
 			break;
 		}
@@ -987,6 +656,7 @@ static double Atracking( double *lasthorizontalw, double *lastverticalw,
 				*--gaptable1 = 'o';
 				*--gaptable2 = '-';
 				k++;
+				*ngap2=1;
 			}
 			l= jin - jfi;
 			while( --l )
@@ -994,6 +664,7 @@ static double Atracking( double *lasthorizontalw, double *lastverticalw,
 				*--gaptable1 = '-';
 				*--gaptable2 = 'o';
 				k++;
+				*ngap1=1;
 			}
 		}
 		if( iin <= 0 || jin <= 0 ) break;
@@ -1003,8 +674,15 @@ static double Atracking( double *lasthorizontalw, double *lastverticalw,
 		iin = ifi; jin = jfi;
 	}
 
-	for( i=0; i<icyc; i++ ) gapireru( mseq1[i], seq1[i], gaptable1 );
-	for( j=0; j<jcyc; j++ ) gapireru( mseq2[j], seq2[j], gaptable2 );
+//	reporterr( "gaptable1=%s\n", gaptable1 );
+//	reporterr( "gaptable2=%s\n", gaptable2 );
+
+
+	if( *ngap1 || !reuseprofiles )
+		for( i=0; i<icyc; i++ ) gapireru( mseq1[i], seq1[i], gaptable1 );
+
+	if( *ngap2 || !reuseprofiles )
+		for( j=0; j<jcyc; j++ ) gapireru( mseq2[j], seq2[j], gaptable2 );
 
 	free( gt1bk );
 	free( gt2bk );
@@ -1012,9 +690,15 @@ static double Atracking( double *lasthorizontalw, double *lastverticalw,
 	return( 0.0 );
 }
 
-double A__align( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1, double *eff2, int icyc, int jcyc, int alloclen, LocalHom ***localhom, double *impmatch, char *sgap1, char *sgap2, char *egap1, char *egap2, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp )
+double A__align( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1, double *eff2, int icyc, int jcyc, int alloclen, LocalHom ***localhom, double *impmatch, char *sgap1, char *sgap2, char *egap1, char *egap2, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp, int firstmem, int calledbyfulltreebase )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
+	int reuseprofiles;
+	static TLS int previousfirstlen; // 2016/Feb/1 // MEMBER NO CHECK GA HITSUYOU!!!!
+	static TLS int previousicyc; // 2016/Feb/1 // MEMBER NO CHECK GA HITSUYOU!!!!
+	static TLS int previousfirstmem;
+	static TLS int previouscall;
+	int ngap1, ngap2;
 //	int k;
 	register int i, j;
 	int lasti, lastj;      /* outgap == 0 -> lgth1, outgap == 1 -> lgth1+1 */
@@ -1043,9 +727,9 @@ double A__align( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1,
 	static TLS char **mseq1;
 	static TLS char **mseq2;
 	static TLS char **mseq;
-	static TLS double *ogcp1;
+	static TLS double *ogcp1, *ogcp1o;
 	static TLS double *ogcp2;
-	static TLS double *fgcp1;
+	static TLS double *fgcp1, *fgcp1o;
 	static TLS double *fgcp2;
 	static TLS double **cpmx1;
 	static TLS double **cpmx2;
@@ -1093,7 +777,7 @@ double A__align( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1,
 			orlgth1 = 0;
 			orlgth2 = 0;
 
-			imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0 );
+			imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, NULL );
 
 			free( mseq1 );
 			free( mseq2 );
@@ -1109,8 +793,10 @@ double A__align( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1,
 			FreeCharMtx( mseq );
 
 			FreeFloatVec( ogcp1 );
+			FreeFloatVec( ogcp1o );
 			FreeFloatVec( ogcp2 );
 			FreeFloatVec( fgcp1 );
+			FreeFloatVec( fgcp1o );
 			FreeFloatVec( fgcp2 );
 
 
@@ -1233,8 +919,10 @@ double A__align( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1,
 			FreeCharMtx( mseq );
 
 			FreeFloatVec( ogcp1 );
+			FreeFloatVec( ogcp1o );
 			FreeFloatVec( ogcp2 );
 			FreeFloatVec( fgcp1 );
+			FreeFloatVec( fgcp1o );
 			FreeFloatVec( fgcp2 );
 
 
@@ -1268,12 +956,16 @@ double A__align( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1,
 		mseq = AllocateCharMtx( njob, ll1+ll2 );
 
 		ogcp1 = AllocateFloatVec( ll1+2 );
+		ogcp1o = AllocateFloatVec( ll1+2 );
 		ogcp2 = AllocateFloatVec( ll2+2 );
 		fgcp1 = AllocateFloatVec( ll1+2 );
+		fgcp1o = AllocateFloatVec( ll1+2 );
 		fgcp2 = AllocateFloatVec( ll2+2 );
 
 		cpmx1 = AllocateFloatMtx( nalphabets, ll1+2 );
 		cpmx2 = AllocateFloatMtx( nalphabets, ll2+2 );
+		previousfirstlen = -1;
+		previousicyc = -1;
 
 		gapfreq1 = AllocateFloatVec( ll1+2 );
 		gapfreq2 = AllocateFloatVec( ll2+2 );
@@ -1343,15 +1035,32 @@ double A__align( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1,
 	}
 #endif
 
-	cpmx_calc_new( seq1, cpmx1, eff1, lgth1, icyc );
+	if( calledbyfulltreebase && previouscall && firstmem >= 0 && firstmem == previousfirstmem && lgth1 == previousfirstlen && nthread == 0 && icyc == previousicyc+1 ) // mouhitotsu jouken ga iru. firstmem >= 0 && firstmem == previousfirstmem && calledbytreebase && prevcalledbytreebase
+		reuseprofiles = 1;
+	else
+		reuseprofiles = 0;
+
+
+	if( reuseprofiles )
+	{
+//		reporterr( "r" );
+		cpmx_calc_add( seq1, cpmx1, eff1, lgth1, icyc );
+	}
+	else
+	{
+//		reporterr( "n" );
+		cpmx_calc_new( seq1, cpmx1, eff1, lgth1, icyc );
+	}
 	cpmx_calc_new( seq2, cpmx2, eff2, lgth2, jcyc );
 
 	if( sgap1 )
 	{
-		new_OpeningGapCount( ogcp1, icyc, seq1, eff1, lgth1, sgap1 );
+		new_OpeningGapCount( ogcp1o, icyc, seq1, eff1, lgth1, sgap1 );
+		new_FinalGapCount( fgcp1o, icyc, seq1, eff1, lgth1, egap1 );
+
 		new_OpeningGapCount( ogcp2, jcyc, seq2, eff2, lgth2, sgap2 );
-		new_FinalGapCount( fgcp1, icyc, seq1, eff1, lgth1, egap1 );
 		new_FinalGapCount( fgcp2, jcyc, seq2, eff2, lgth2, egap2 );
+
 		outgapcount( &headgapfreq1, icyc, sgap1, eff1 );
 		outgapcount( &headgapfreq2, jcyc, sgap2, eff2 );
 		outgapcount( gapfreq1+lgth1, icyc, egap1, eff1 );
@@ -1359,10 +1068,20 @@ double A__align( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1,
 	}
 	else
 	{
-		st_OpeningGapCount( ogcp1, icyc, seq1, eff1, lgth1 );
+		if( reuseprofiles )
+		{
+			st_OpeningGapAdd( ogcp1o, icyc, seq1, eff1, lgth1 );
+			st_FinalGapAdd( fgcp1o, icyc, seq1, eff1, lgth1 );
+		}
+		else
+		{
+			st_OpeningGapCount( ogcp1o, icyc, seq1, eff1, lgth1 );
+			st_FinalGapCount( fgcp1o, icyc, seq1, eff1, lgth1 );
+		}
+
 		st_OpeningGapCount( ogcp2, jcyc, seq2, eff2, lgth2 );
-		st_FinalGapCount( fgcp1, icyc, seq1, eff1, lgth1 );
 		st_FinalGapCount( fgcp2, jcyc, seq2, eff2, lgth2 );
+
 		headgapfreq1 = 0.0;
 		headgapfreq2 = 0.0;
 		gapfreq1[lgth1] = 0.0;
@@ -1371,8 +1090,13 @@ double A__align( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1,
 
 	if( legacygapcost == 0 )
 	{
-		gapcountf( gapfreq1, seq1, icyc, eff1, lgth1 );
+		if( reuseprofiles )
+			gapcountadd( gapfreq1, seq1, icyc, eff1, lgth1 );
+		else
+			gapcountf( gapfreq1, seq1, icyc, eff1, lgth1 );
+
 		gapcountf( gapfreq2, seq2, jcyc, eff2, lgth2 );
+
 		for( i=0; i<lgth1+1; i++ ) gapfreq1[i] = 1.0 - gapfreq1[i];
 		for( i=0; i<lgth2+1; i++ ) gapfreq2[i] = 1.0 - gapfreq2[i];
 		headgapfreq1 = 1.0 - headgapfreq1;
@@ -1399,8 +1123,8 @@ double A__align( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1,
 
 	for( i=0; i<lgth1; i++ ) 
 	{
-		ogcp1[i] = 0.5 * ( 1.0 - ogcp1[i] ) * fpenalty * ( gapfreq1[i] );
-		fgcp1[i] = 0.5 * ( 1.0 - fgcp1[i] ) * fpenalty * ( gapfreq1[i] );
+		ogcp1[i] = 0.5 * ( 1.0 - ogcp1o[i] ) * fpenalty * ( gapfreq1[i] );
+		fgcp1[i] = 0.5 * ( 1.0 - fgcp1o[i] ) * fpenalty * ( gapfreq1[i] );
 	}
 
 	for( i=0; i<lgth2; i++ ) 
@@ -1737,10 +1461,12 @@ fprintf( stderr, "\n" );
 	*/
 	if( localhom )
 	{
-		Atracking_localhom( impmatch, currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, icyc, jcyc, warpis, warpjs, warpbase );
+		Atracking_localhom( impmatch, currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, icyc, jcyc, warpis, warpjs, warpbase, &ngap1, &ngap2, reuseprofiles );
 	}
 	else
-		Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, icyc, jcyc, tailgp, warpis, warpjs, warpbase );
+		Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, icyc, jcyc, tailgp, warpis, warpjs, warpbase, &ngap1, &ngap2, reuseprofiles );
+
+//	reporterr( "reuseprofiles after Atracking = %d\n", reuseprofiles );
 
 	if( warpis ) free( warpis );
 	if( warpjs ) free( warpjs );
@@ -1755,8 +1481,11 @@ fprintf( stderr, "\n" );
 	}
 
 
-	for( i=0; i<icyc; i++ ) strcpy( seq1[i], mseq1[i] );
-	for( j=0; j<jcyc; j++ ) strcpy( seq2[j], mseq2[j] );
+	if( ngap1 || !reuseprofiles )
+		for( i=0; i<icyc; i++ ) strcpy( seq1[i], mseq1[i] );
+	if( ngap2 || !reuseprofiles )
+		for( j=0; j<jcyc; j++ ) strcpy( seq2[j], mseq2[j] );
+
 #if 0
 	fprintf( stderr, "\n" );
 	for( i=0; i<icyc; i++ ) fprintf( stderr, "%s\n", mseq1[i] );
@@ -1764,7 +1493,14 @@ fprintf( stderr, "\n" );
 	for( j=0; j<jcyc; j++ ) fprintf( stderr, "%s\n", mseq2[j] );
 #endif
 
-//	fprintf( stderr, "wm = %f\n", wm );
+//	fprintf( stdout, "firstmem=%d, icyc=%d, jcyc=%d, wm = %f\n", firstmem, icyc, jcyc, wm );
+
+//	fprintf( stderr, "lgth1 = %d\n", lgth1 );
+//	fprintf( stderr, "->      %d\n", strlen( seq1[0] ) );
+	previousfirstlen = lgth1;
+	previousfirstmem = firstmem;
+	previousicyc = icyc;
+	previouscall = calledbyfulltreebase;
 
 	return( wm );
 }
@@ -1784,6 +1520,7 @@ double A__align_variousdist( int **which, double ***matrices, double **n_dynamic
 
 //	int k;
 	register int i, j, c;
+	int ngap1, ngap2;
 	int lasti, lastj;      /* outgap == 0 -> lgth1, outgap == 1 -> lgth1+1 */
 	int lgth1, lgth2;
 	int resultlen;
@@ -1858,7 +1595,7 @@ double A__align_variousdist( int **which, double ***matrices, double **n_dynamic
 			orlgth1 = 0;
 			orlgth2 = 0;
 
-			imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0 );
+			imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, NULL );
 
 			free( mseq1 );
 			free( mseq2 );
@@ -1924,7 +1661,7 @@ double A__align_variousdist( int **which, double ***matrices, double **n_dynamic
 		}
 	}
 	for( c=0; c<maxdistclass; c++ ) if( nmask[c] ) break;
-	if( c<maxdistclass ) reporterr( "Found a complex grouping. This step may be a bit slow.\n", c );
+	if( c<maxdistclass ) reporterr( "Found a complex grouping. This step may be a bit slow.\n" );
 #endif
 
 	lgth1 = strlen( seq1[0] );
@@ -2586,10 +2323,10 @@ fprintf( stderr, "\n" );
 	*/
 	if( localhom )
 	{
-		Atracking_localhom( impmatch, currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, icyc, jcyc, warpis, warpjs, warpbase );
+		Atracking_localhom( impmatch, currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, icyc, jcyc, warpis, warpjs, warpbase, &ngap1, &ngap2, 0 );
 	}
 	else
-		Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, icyc, jcyc, tailgp, warpis, warpjs, warpbase );
+		Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, icyc, jcyc, tailgp, warpis, warpjs, warpbase, &ngap1, &ngap2, 0 );
 
 	if( warpis ) free( warpis );
 	if( warpjs ) free( warpjs );
