@@ -21,7 +21,6 @@ static int mapout;
 static int smoothing;
 static double hitout;
 
-static double lenfaca, lenfacb, lenfacc, lenfacd;
 static int tuplesize;
 
 #define PLENFACA 0.01
@@ -862,6 +861,7 @@ void arguments( int argc, char *argv[] )
 	smoothing = 0;
 	distout = 0;
 	hitout = 0.0;
+	nwildcard = 0;
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -1115,6 +1115,9 @@ void arguments( int argc, char *argv[] )
 				case 'Z':
 					mapout = 1;
 					break;
+				case ':':
+					nwildcard = 1;
+					break;
                 default:
                     fprintf( stderr, "illegal option %c\n", c );
                     argc = 0;
@@ -1393,9 +1396,9 @@ static double treebase( int nseq, int *nlen, char **aseq, int nadd, char *mergeo
 			fprintf( stderr, "c" );
 			if( alg == 'A' )
 			{
-				imp_match_init_strict( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+				imp_match_init_strict( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, NULL, 1, topol[l][0], topol[l][1] );
 				if( rnakozo ) imp_rna( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, NULL, NULL, NULL );
-				pscore = A__align( n_dis_consweight_multi, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
+				pscore = A__align( n_dis_consweight_multi, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap, -1, -1 );
 			}
 			else if( alg == 'Q' )
 			{
@@ -1428,7 +1431,7 @@ static double treebase( int nseq, int *nlen, char **aseq, int nadd, char *mergeo
 					pscore = MSalignmm( n_dis_consweight_multi, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					break;
 				case( 'A' ):
-					pscore = A__align( n_dis_consweight_multi, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
+					pscore = A__align( n_dis_consweight_multi, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap, -1, -1 );
 					break;
 				default:
 					ErrorExit( "ERROR IN SOURCE FILE" );
@@ -1972,7 +1975,7 @@ static void	*addsinglethread( void *arg )
 		if( mseq1 ) free( mseq1 ); mseq1 = NULL;
 		if( mseq2 ) free( mseq2 ); mseq2 = NULL;
 		Falign( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
-		A__align( NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
+		A__align( NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0, -1, -1 );
 		if( commonIP ) FreeIntMtx( commonIP );
 		commonIP = NULL;
 		commonAlloc1 = commonAlloc2 = 0;
@@ -1982,8 +1985,6 @@ static void	*addsinglethread( void *arg )
 		return( NULL );
 	}
 
-static int maxl;
-static int tsize;
 static int nunknown = 0;
 
 void seq_grp_nuc( int *grp, char *seq )
@@ -2036,49 +2037,6 @@ void makecompositiontable_p( int *table, int *pointt )
 		table[point]++;
 }
 
-int commonsextet_p( int *table, int *pointt )
-{
-	int value = 0;
-	int tmp;
-	int point;
-	static TLS int *memo = NULL;
-	static TLS int *ct = NULL;
-	int *cp;
-
-	if( table == NULL )
-	{
-		if( memo ) free( memo );
-		if( ct ) free( ct );
-		return( 0 );
-	}
-
-	if( *pointt == -1 )
-		return( 0 );
-
-	if( !memo )
-	{
-		memo = (int *)calloc( tsize, sizeof( int ) );
-		if( !memo ) ErrorExit( "Cannot allocate memo\n" );
-		ct = (int *)calloc( MIN( maxl, tsize )+1, sizeof( int ) ); // chuui!!
-		if( !ct ) ErrorExit( "Cannot allocate ct\n" );
-	}
-
-	cp = ct;
-	while( ( point = *pointt++ ) != END_OF_VEC )
-	{
-		tmp = memo[point]++;
-		if( tmp < table[point] )
-			value++;
-		if( tmp == 0 ) *cp++ = point;
-	}
-	*cp = END_OF_VEC;
-	
-	cp =  ct;
-	while( *cp != END_OF_VEC )
-		memo[*cp++] = 0;
-
-	return( value );
-}
 
 void makepointtable_nuc_dectet( int *pointt, int *n )
 {
