@@ -22,6 +22,29 @@ static int shishagonyuu( double in )
 	return( out );
 }
 
+static void nscore( int *amino_n, int **n_dis )
+{
+	int i;
+	for( i=0; i<26; i++ )
+	{
+//		reporterr( "i=%d (%c), n_dis[%d][%d] = %d\n", i, amino[i], i, amino_n['n'], n_dis[i][amino_n['n']] );
+		n_dis[i][amino_n['n']] = shishagonyuu( (double)0.25 * n_dis[i][i] );
+//		reporterr( "-> i=%d, n_dis[%d][%d] = %d\n", i, i, amino_n['n'], n_dis[i][amino_n['n']] );
+		n_dis[amino_n['n']][i] = n_dis[i][amino_n['n']];
+	}
+	n_dis[amino_n['n']][amino_n['n']] = shishagonyuu( (double)0.25 * 0.25 * ( n_dis[0][0] + n_dis[1][1] + n_dis[2][2] + n_dis[3][3] ) );
+
+#if 0 // Ato de kakunin
+	for( i=0; i<26; i++ )
+	{
+		n_dis[i][amino_n['-']] = shishagonyuu( (double)0.25 * n_dis[i][i] );
+		n_dis[amino_n['-']][i] = n_dis[i][amino_n['-']];
+	}
+//	n_dis[amino_n['-']][amino_n['-']] = shishagonyuu( (double)0.25 * 0.25 * ( n_dis[0][0] + n_dis[1][1] + n_dis[2][2] + n_dis[3][3] ) ); // DAME!
+#endif
+}
+
+
 static void ambiguousscore( int *amino_n, int **n_dis )
 {
 	int i;
@@ -457,6 +480,7 @@ void constants( int nseq, char **seq )
 		for( i=0; i<10; i++ ) for( j=0; j<10; j++ ) n_dis[i][j] = shishagonyuu( pamx[i][j] );
 
 		ambiguousscore( amino_n, n_dis );
+		if( nwildcard ) nscore( amino_n, n_dis );
 
         if( disp )
         {
@@ -469,6 +493,7 @@ void constants( int nseq, char **seq )
             }
             reporterr(       "\n" );
 			reporterr(       "penalty = %d, penalty_ex = %d\n", penalty, penalty_ex );
+//exit( 1 );
         }
 
 // RIBOSUM
@@ -537,7 +562,7 @@ void constants( int nseq, char **seq )
        		}
        		reporterr(       "\n" );
       	}
-		reporterr(       "done\n" );
+//		reporterr(       "done\n" );
 
 #if 1
 		for( i=0; i<37; i++ ) for( j=0; j<37; j++ ) ribosumdis[i][j] = 0.0; //iru
@@ -564,7 +589,7 @@ void constants( int nseq, char **seq )
        		}
        		reporterr(       "\n" );
       	}
-		reporterr(       "done\n" );
+//		reporterr(       "done\n" );
 #endif
 
 		FreeDoubleMtx( pam1 );
@@ -763,7 +788,7 @@ void constants( int nseq, char **seq )
 		FreeDoubleVec( datafreq );
 		FreeDoubleVec( freq );
 
-		reporterr(       "done.\n" );
+//		reporterr(       "done.\n" );
 
 	}
 	else if( dorp == 'p' && scoremtx == 1 )  /* Blosum, user-defined */
@@ -783,11 +808,11 @@ void constants( int nseq, char **seq )
 			reporterr( "nblosum=%d??\n", nblosum );
 			exit( 1 );
 		}
-		if( nblosum < 0 )
-		{
-			nblosum *= -1;
-			makeaverage0 = 0;
-		}
+//		if( nblosum < 0 )
+//		{
+//			nblosum *= -1;
+//			makeaverage0 = 0;
+//		}
 		else
 		{
 			makeaverage0 = 1;
@@ -971,7 +996,7 @@ void constants( int nseq, char **seq )
 		FreeDoubleVec( datafreq );
 		FreeDoubleVec( freq );
 
-		reporterr(       "done.\n" );
+//		reporterr(       "done.\n" );
 
 	}
 	else if( dorp == 'p' && scoremtx == 2 ) /* Miyata-Yasunaga */
@@ -1299,7 +1324,6 @@ void constants( int nseq, char **seq )
 		FreeDoubleVec( mutab );
 		FreeDoubleVec( datafreq );
 	}
-	reporterr(       "scoremtx = %d\n", scoremtx );
 
 #if DEBUG
 	reporterr(       "scoremtx = %d\n", scoremtx );
@@ -1309,7 +1333,7 @@ void constants( int nseq, char **seq )
 	amino_dis = AllocateIntMtx( charsize, charsize );
 	amino_dis_consweight_multi = AllocateDoubleMtx( charsize, charsize );
 
-	reporterr( "charsize=%d\n", charsize );
+//	reporterr( "charsize=%d\n", charsize );
 
 	for( i=0; i<charsize; i++ )amino_n[i] = -1;
 	for( i=0; i<nalphabets; i++) amino_n[(int)amino[i]] = i;
@@ -1445,4 +1469,6 @@ void freeconstants()
 	FreeIntMtx( n_dis );
 	FreeIntMtx( n_disFFT );
 	FreeDoubleMtx( n_dis_consweight_multi );
+	FreeIntMtx( amino_dis );
+	FreeDoubleMtx( amino_dis_consweight_multi );
 }
