@@ -38,7 +38,7 @@ int fmodel; // 1-> fmodel 0->default -1->raw
 int nblosum; // 45, 50, 62, 80
 int kobetsubunkatsu;
 int bunkatsu;
-int dorp;
+int dorp = NOTSPECIFIED; // arguments de shitei suruto, tbfast -> pairlocalalign no yobidashi de futsugou
 int niter;
 int contin;
 int calledByXced;
@@ -103,6 +103,7 @@ int spscoreout = 0;
 int outnumber = 0;
 int legacygapcost = 0;
 double minimumweight = 0.0005;
+int nwildcard = 0;
 
 char *signalSM;
 FILE *prep_g;
@@ -127,6 +128,9 @@ int gmsg = 0;
 
 double sueff_global = SUEFF;
 
+double lenfaca, lenfacb, lenfacc, lenfacd;
+int maxl, tsize;
+
 void initglobalvariables()
 {
 	commonAlloc1 = 0;
