@@ -83,7 +83,13 @@ void *athread( void *arg )
 				mtxv = maxdist * ( 1.0 - (double)naivepairscorefast( seq[i], seq[j], skiptable[i], skiptable[j], penalty ) / bunbo );
 		}
 #if 1
-		if( mtxv > 9.0 || mtxv < 0.0 )
+		if( mtxv < 0.0 )
+		{
+			reporterr( "WARNING: negative distance, mtxv = %f\n", mtxv );
+			mtxv = 0.0;
+		}
+
+		if( mtxv > 9.0 )
 		{
 			fprintf( stderr, "Distance %d-%d is strange, %f.\n", i, j, mtxv );
 			exit( 1 );
@@ -124,6 +130,7 @@ void arguments( int argc, char *argv[] )
 	kimuraR = NOTSPECIFIED;
 	pamN = NOTSPECIFIED;
 	usenaivescoreinsteadofalignmentscore = 0;
+	nwildcard = 0;
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -146,6 +153,9 @@ void arguments( int argc, char *argv[] )
 				case 'P':
 					dorp = 'p';
 					break;
+				case ':':
+					nwildcard = 1;
+					break;
 				case 'K': // Hontou ha iranai. disttbfast.c, tbfast.c to awaserutame.
 					break;
 				case 'I':
@@ -180,29 +190,29 @@ void arguments( int argc, char *argv[] )
 					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
-					fprintf( stderr, "jtt %d\n", pamN );
+//					fprintf( stderr, "jtt %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'm':
 					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = TM;
-					fprintf( stderr, "TM %d\n", pamN );
+//					fprintf( stderr, "TM %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'i':
 					inputfile = *++argv;
-					fprintf( stderr, "inputfile = %s\n", inputfile );
+//					fprintf( stderr, "inputfile = %s\n", inputfile );
 					--argc;
 					goto nextoption;
 				case 'M':
 					maxdist = myatoi( *++argv );
-					fprintf( stderr, "maxdist = %d\n", maxdist );
+//					fprintf( stderr, "maxdist = %d\n", maxdist );
 					--argc; 
 					goto nextoption;
 				case 'C':
 					nthread = myatoi( *++argv );
-					fprintf( stderr, "nthread = %d\n", nthread );
+//					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
 					goto nextoption;
 					break;
@@ -372,6 +382,11 @@ int main( int argc, char **argv )
 
 				}
 #if 1
+				if( mtxv < 0.0 )
+				{
+					reporterr( "WARNING: negative distance, mtxv = %f\n", mtxv );
+					mtxv = 0.0;
+				}
 				if( mtxv > 9.0 || mtxv < 0.0 )
 				{
 					fprintf( stderr, "Distance %d-%d is strange, %f.\n", i, j, mtxv );
