@@ -7,6 +7,7 @@ extern char **seq_g;
 extern char **res_g;
 static int subalignment;
 static int subalignmentoffset;
+static int specifictarget;
 
 static int intop;
 static int intree;
@@ -101,6 +102,8 @@ void arguments( int argc, char *argv[] )
 	legacygapcost = 0;
 	specificityconsideration = 0.0;
 	autosubalignment = 0.0;
+	specifictarget = 0;
+	nwildcard = 0;
 
 	while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -376,6 +379,12 @@ void arguments( int argc, char *argv[] )
 					fftWinSize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
+				case '=':
+					specifictarget = 1;
+					break;
+				case ':':
+					nwildcard = 1;
+					break;
 				default:
 					fprintf( stderr, "illegal option %c\n", c );
 					argc = 0;
@@ -436,6 +445,8 @@ int main( int argc, char *argv[] )
 	int *insubtable;
 	int *preservegaps;
 	char ***subalnpt;
+	int ntarget, *targetmap, *targetmapr;
+	int ilim;
 
 	arguments( argc, argv );
 #ifndef enablemultithread
@@ -515,13 +526,45 @@ int main( int argc, char *argv[] )
 	skipthisbranch = AllocateIntMtx( njob, 2 );
 	for( i=0; i<njob; i++ ) skipthisbranch[i][0] = skipthisbranch[i][1] = 0;
 
+
+#if 0
+	Read( name, nlen, seq_g );
+#else
+	readData_pointer( infp, name, nlen, seq_g );
+#endif
+	fclose( infp );
+
+	if( specifictarget )
+	{
+		targetmap = calloc( njob, sizeof( int ) );
+		ntarget = 0;
+		for( i=0; i<njob; i++ )
+		{
+			targetmap[i] = -1;
+			if( !strncmp( name[i]+1, "_focus_", 7 ) )
+				targetmap[i] = ntarget++;
+		}
+		targetmapr = calloc( ntarget, sizeof( int ) );
+		for( i=0; i<njob; i++ )
+			if( targetmap[i] != -1 ) targetmapr[targetmap[i]] = i;
+	}
+	else
+	{
+		ntarget = njob;
+		targetmap = calloc( njob, sizeof( int ) );
+		targetmapr = calloc( njob, sizeof( int ) );
+		for( i=0; i<njob; i++ )
+			targetmap[i] = targetmapr[i] = i;
+	}
+
 	if( constraint )
 	{
-		localhomtable = (LocalHom **)calloc( njob, sizeof( LocalHom *) );
-		for( i=0; i<njob; i++)
+		ilim = njob;
+		localhomtable = (LocalHom **)calloc( ntarget, sizeof( LocalHom *) );
+		for( i=0; i<ntarget; i++)
 		{
-			localhomtable[i] = (LocalHom *)calloc( njob, sizeof( LocalHom ) );
-			for( j=0; j<njob; j++)
+			localhomtable[i] = (LocalHom *)calloc( ilim, sizeof( LocalHom ) );
+			for( j=0; j<ilim; j++)
 			{
 				localhomtable[i][j].start1 = -1;
 				localhomtable[i][j].end1 = -1;
@@ -536,12 +579,14 @@ int main( int argc, char *argv[] )
 				localhomtable[i][j].last = localhomtable[i]+j;
 				localhomtable[i][j].korh = 'h';
 			}
+			if( !specifictarget ) ilim--;
 		}
 		fprintf( stderr, "Loading 'hat3' ... " );
 		fflush( stderr );
 		prep = fopen( "hat3", "r" );
 		if( prep == NULL ) ErrorExit( "Make hat3." );
-		readlocalhomtable2( prep, njob, localhomtable, kozoarivec );
+		if( specifictarget ) readlocalhomtable2_target( prep, njob, localhomtable, kozoarivec, targetmap );
+		else readlocalhomtable2_half( prep, njob, localhomtable, kozoarivec );
 		fclose( prep ); 
 //		for( i=0; i<njob-1; i++ ) for( j=i+1; j<njob; j++ )
 //			fprintf( stdout, "%d %d %d %d %d %d %d\n", i, j, localhomtable[i][j].opt, localhomtable[i][j].start1, localhomtable[i][j].end1, localhomtable[i][j].start2, localhomtable[i][j].end2 );
@@ -579,12 +624,6 @@ int main( int argc, char *argv[] )
 			}
 		}
 
-#if 0
-	Read( name, nlen, seq_g );
-#else
-	readData_pointer( infp, name, nlen, seq_g );
-#endif
-	fclose( infp );
 
 	if( specificityconsideration ) calcmaxdistclass();
 
@@ -989,7 +1028,7 @@ exit( 1 );
 		fprintf( trap_g, "Segment %3d/%3d %4d-%4d\n", iseg+1, nseg-1, pos+1, pos+1+tmplen );
 	
 		cut = ocut;
-		returnvalue = TreeDependentIteration( njob, name, nlen, seq, bseq, topol, len, eff, skipthisbranch, alloclen, localhomtable, singlerna, nkozo, kozoarivec );
+		returnvalue = TreeDependentIteration( njob, name, nlen, seq, bseq, topol, len, eff, skipthisbranch, alloclen, localhomtable, singlerna, nkozo, kozoarivec, ntarget, targetmap, targetmapr );
 
 		for( i=0; i<njob; i++ )
 			strcat( res_g[i], bseq[i] );
@@ -1000,7 +1039,13 @@ exit( 1 );
 	FreeDoubleMtx( eff );
 	FreeIntMtx( skipthisbranch );
 	free( kozoarivec );
-	if( constraint ) FreeLocalHomTable( localhomtable, njob );
+	if( constraint ) 
+	{
+		if( specifictarget ) FreeLocalHomTable_part( localhomtable, ntarget, njob );
+		else FreeLocalHomTable_half( localhomtable, njob );
+	}
+	free( targetmap );
+	free( targetmapr );
 	if( rnakozo && rnaprediction == 'm' ) 
 	{
 		if( singlerna ) // nen no tame
