@@ -164,8 +164,6 @@ void arguments( int argc, char *argv[] )
 
 
 
-static int maxl;
-static int tsize;
 
 void seq_grp_nuc( int *grp, char *seq )
 {
@@ -280,7 +278,7 @@ void makepointtable( int *pointt, int *n )
 	*pointt = END_OF_VEC;
 }
 
-static int commonsextet_p2( short *table, int *pointt )
+static int localcommonsextet_p2( short *table, int *pointt )
 {
 	int value = 0;
 	short tmp;
@@ -379,7 +377,7 @@ static void	*directionthread( void *arg )
 			res[j] = L__align11_noalign( n_dis_consweight_multi, mseq1, mseq2 );
 		}
 		else
-			res[j] = commonsextet_p2( table1, spointt[j] );
+			res[j] = localcommonsextet_p2( table1, spointt[j] );
 	}
 	if( dodp ) // nakuserukamo
 	{
@@ -390,7 +388,7 @@ static void	*directionthread( void *arg )
 	}
 //	else
 //		if( nthread )  // inthread == 0 no toki free suru to, error. nazeda
-//			commonsextet_p( NULL, NULL );
+//			localcommonsextet_p( NULL, NULL );
 	return( NULL );
 }
 
