@@ -22,8 +22,8 @@
 #include <pthread.h>
 #endif
 
-#define VERSION "7.271"
-#define SHOWVERSION reporterr( "%s (%s) Version " VERSION " alg=%c, model=%s, amax=%3.1f\n%d thread(s)\n", progName( argv[0] ), (dorp=='d')?"nuc":((nblosum==-2)?"text":"aa"), alg, modelname, specificityconsideration, nthread )
+#define VERSION "7.294b"
+#define SHOWVERSION reporterr( "%s (%s) Version " VERSION " alg=%c, model=%s, amax=%3.1f\n%d thread(s)\n\n", progName( argv[0] ), (dorp=='d')?"nuc":((nblosum==-2)?"text":"aa"), alg, modelname, specificityconsideration, nthread )
 
 #define FFT_THRESHOLD  80
 #define FFT_WINSIZE_P   20
@@ -197,8 +197,9 @@ typedef struct _LocalHom
 	int overlapaa;
 	int extended;
 	double importance;
-	double fimportance;
-	double wimportance;
+	double rimportance;
+//	double fimportance;
+//	double wimportance;
 	char korh;
 } LocalHom;
 
@@ -319,6 +320,7 @@ extern int outnumber;
 
 extern int legacygapcost;
 extern double minimumweight;
+extern int nwildcard;
 
 extern TLS char *newgapstr;
 
@@ -330,3 +332,5 @@ extern int ndistclass, maxdistclass;
 extern int gmsg;
 
 extern double sueff_global;
+extern double lenfaca, lenfacb, lenfacc, lenfacd;
+int maxl, tsize;
