@@ -26,14 +26,11 @@ static int uselongest;
 static int treeout;
 static int classsize;
 static int picksize;
-static int maxl;
-static int tsize;
 static int reorder;
 static int pid;
 static int maxdepth = 0;
 static double tokyoripara;
 
-static double lenfaca, lenfacb, lenfacc, lenfacd;
 #define PLENFACA 0.01
 #define PLENFACB 10000
 #define PLENFACC 10000
@@ -798,8 +795,6 @@ void arguments( int argc, char *argv[] )
 	}
 }
 
-static int maxl;
-static int tsize;
 static int nunknown = 0;
 
 int seq_grp_nuc( int *grp, char *seq )
@@ -842,7 +837,7 @@ void makecompositiontable_p( short *table, int *pointt )
 		table[point]++;
 }
 
-int commonsextet_p( short *table, int *pointt )
+static int localcommonsextet_p( short *table, int *pointt )
 {
 	int value = 0;
 	short tmp;
@@ -935,7 +930,7 @@ static void pairalign( int nseq, int *nlen, char **seq, int *mem1, int *mem2, do
 	static double *effarr1 = NULL;
 	static double *effarr2 = NULL;
 	static char **mseq1, **mseq2;
-	double dumfl = 0.0;
+//	double dumfl = 0.0;
 	double dumdb = 0.0;
 	int ffttry;
 	int m1, m2;
@@ -1047,7 +1042,7 @@ static void pairalign( int nseq, int *nlen, char **seq, int *mem1, int *mem2, do
 				else
 				{
 //					fprintf( stderr, "%d-%d", clus1, clus2 );
-					pscore = A__align( n_dis_consweight_multi, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
+					pscore = A__align( n_dis_consweight_multi, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap, -1, -1 );
 				}
 				break;
 			default:
@@ -1421,7 +1416,7 @@ static int splitseq_mq( Scores *scores, int nin, int *nlen, char **seq, char **o
 			}
 			else
 			{
-				scores[i].score = ( 1.0 - (double)commonsextet_p( table1, scores[i].pointt ) / MIN( selfscore0, scores[i].selfscore ) ) * lenfac;
+				scores[i].score = ( 1.0 - (double)localcommonsextet_p( table1, scores[i].pointt ) / MIN( selfscore0, scores[i].selfscore ) ) * lenfac;
 				if( scores[i].score > MAX6DIST ) scores[i].score = MAX6DIST;
 			}
 //			if( i ) fprintf( stderr, "%d-%d d %4.2f len %d %d\n", 1, i+1, scores[i].score, scores->orilen, scores[i].orilen );
@@ -1682,7 +1677,7 @@ exit( 1 );
 			}
 			else
 			{
-				pickmtx[j][i-j] = ( 1.0 - (double)commonsextet_p( table1, scores[picks[i]].pointt ) / MIN( selfscore0, scores[picks[i]].selfscore ) ) * lenfac;
+				pickmtx[j][i-j] = ( 1.0 - (double)localcommonsextet_p( table1, scores[picks[i]].pointt ) / MIN( selfscore0, scores[picks[i]].selfscore ) ) * lenfac;
 				if( pickmtx[j][i-j] > MAX6DIST ) pickmtx[j][i-j] = MAX6DIST;
 			}
 
@@ -2200,7 +2195,7 @@ exit( 1 );
 					}
 					else
 					{
-						dfromc[i][j] = ( 1.0 - (double)commonsextet_p( table1, scores[j].pointt ) / MIN( selfscore0, scores[j].selfscore ) ) * lenfac;
+						dfromc[i][j] = ( 1.0 - (double)localcommonsextet_p( table1, scores[j].pointt ) / MIN( selfscore0, scores[j].selfscore ) ) * lenfac;
 						if( dfromc[i][j] > MAX6DIST ) dfromc[i][j] = MAX6DIST;
 					}
 				}
@@ -2991,7 +2986,7 @@ int main( int argc, char *argv[] )
 			table1 = (short *)calloc( tsize, sizeof( short ) );
 			if( !table1 ) ErrorExit( "Cannot allocate table1\n" );
 			makecompositiontable_p( table1, pointt[i] );
-			scores[i].selfscore = commonsextet_p( table1, pointt[i] );
+			scores[i].selfscore = localcommonsextet_p( table1, pointt[i] );
 			free( table1 );
 		}
 	}
