@@ -3,6 +3,7 @@
 #define DEBUG 0
 #define IODEBUG 0
 #define SCOREOUT 0
+#define SHISHAGONYU 0 // for debug
 
 static int nadd;
 static int treein;
@@ -17,6 +18,9 @@ static int keeplength;
 static int ndeleted;
 static int mapout;
 static int smoothing;
+static int specifictarget;
+static int callpairlocalalign;
+static int outputhat23;
 
 #ifdef enablemultithread
 typedef struct _jobtable
@@ -54,14 +58,16 @@ typedef struct _treebasethread_arg
 	double *effarr_kozo;
 	int *fftlog;
 	char *mergeoralign;
+	int *targetmap;
 	pthread_mutex_t *mutex;
 	pthread_cond_t *treecond;
 } treebasethread_arg_t;
 #endif
 
-void arguments( int argc, char *argv[] )
+static void arguments( int argc, char *argv[], int *pac, char **pav, int *tac, char **tav ) // 2 kai yobaremasu.
 {
     int c;
+	int i;
 
 	nthread = 1;
 	outnumber = 0;
@@ -106,7 +112,7 @@ void arguments( int argc, char *argv[] )
 	contin = 0;
 	scoremtx = 1;
 	kobetsubunkatsu = 0;
-	dorp = NOTSPECIFIED;
+//	dorp = NOTSPECIFIED;
 	ppenalty_dist = NOTSPECIFIED;
 	ppenalty = NOTSPECIFIED;
 	penalty_shift_factor = 1000.0;
@@ -131,21 +137,79 @@ void arguments( int argc, char *argv[] )
 	keeplength = 0;
 	mapout = 0;
 	smoothing = 0;
+	specifictarget = 0;
+	callpairlocalalign = 0;
+	outputhat23 = 0;
+	nwildcard = 0;
+
+	if( pac )
+	{
+		pav[0] = "tbfast-pair";
+		*pac = 1;
+		tav[0] = "tbfast";
+		*tac = 1;
+	
+		for( i=0; i<argc; i++ )
+		{
+			if( argv[i][0] == '_' )
+			{
+				callpairlocalalign = 1;
+//				reporterr( "start\n" );
+	
+				for( i++; i<argc; i++ )
+				{
+					if( argv[i][0] == '_' )
+					{
+//						reporterr( "end\n" );
+						goto pavend;
+					}
+					pav[*pac] = argv[i];
+					*pac += 1;
+//					reporterr( "%s\n", argv[i] );
+				}
+			}
+		}
+	
+	
+		pavend:
+
+//		reporterr( "i=%d\n", i );
+		for( i++; i<argc; i++ )
+		{
+			tav[*tac] = argv[i];
+			*tac += 1;
+		}
+
+		argc -= *pac + 1;
+		argv += *pac + 1;
+	
+//		reporterr( "argc in tbfast = %d\n", argc );
+//		reporterr( "*pac in tbfast = %d\n", *pac );
+//		for( i=0; i<*tac; i++ ) reporterr( "%s\n", tav[i] );
+	}
+	else
+	{
+//		reporterr( "SECOND TIME\n" );
+	}
+
+//	reporterr( "*argv = %s\n", *argv );
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
+//		reporterr( "(*argv)[0] = %s\n", (*argv) );
         while ( ( c = *++argv[0] ) )
 		{
+//			reporterr( "c=%c\n", c );
             switch( c )
             {
 				case 'i':
 					inputfile = *++argv;
-					fprintf( stderr, "inputfile = %s\n", inputfile );
+//					fprintf( stderr, "inputfile = %s\n", inputfile );
 					--argc;
                     goto nextoption;
 				case 'I':
 					nadd = myatoi( *++argv );
-					fprintf( stderr, "nadd = %d\n", nadd );
+//					fprintf( stderr, "nadd = %d\n", nadd );
 					--argc;
 					goto nextoption;
 				case 'e':
@@ -172,7 +236,7 @@ void arguments( int argc, char *argv[] )
 					goto nextoption;
 				case 'g':
 					ppenalty_ex = (int)( atof( *++argv ) * 1000 - 0.5 );
-					fprintf( stderr, "ppenalty_ex = %d\n", ppenalty_ex );
+//					fprintf( stderr, "ppenalty_ex = %d\n", ppenalty_ex );
 					--argc;
 					goto nextoption;
 				case 'h':
@@ -182,27 +246,27 @@ void arguments( int argc, char *argv[] )
 					goto nextoption;
 				case 'k':
 					kimuraR = myatoi( *++argv );
-					fprintf( stderr, "kappa = %d\n", kimuraR );
+//					fprintf( stderr, "kappa = %d\n", kimuraR );
 					--argc;
 					goto nextoption;
 				case 'b':
 					nblosum = myatoi( *++argv );
 					scoremtx = 1;
-					fprintf( stderr, "blosum %d / kimura 200\n", nblosum );
+//					fprintf( stderr, "blosum %d / kimura 200\n", nblosum );
 					--argc;
 					goto nextoption;
 				case 'j':
 					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
-					fprintf( stderr, "jtt/kimura %d\n", pamN );
+//					fprintf( stderr, "jtt/kimura %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'm':
 					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = TM;
-					fprintf( stderr, "tm %d\n", pamN );
+//					fprintf( stderr, "tm %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'l':
@@ -221,8 +285,11 @@ void arguments( int argc, char *argv[] )
 					goto nextoption;
 				case 'C':
 					nthread = myatoi( *++argv );
-					fprintf( stderr, "nthread = %d\n", nthread );
+//					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
+#ifndef enablemultithread
+					nthread = 0;
+#endif
 					goto nextoption;
 				case 's':
 					specificityconsideration = (double)myatof( *++argv );
@@ -297,7 +364,7 @@ void arguments( int argc, char *argv[] )
 				case 'X':
 					treemethod = 'X';
 					sueff_global = atof( *++argv );
-					fprintf( stderr, "sueff_global = %f\n", sueff_global );
+//					fprintf( stderr, "sueff_global = %f\n", sueff_global );
 					--argc;
 					goto nextoption;
 				case 'E':
@@ -379,7 +446,7 @@ void arguments( int argc, char *argv[] )
 					goto nextoption;
 				case 'W':
 					minimumweight = atof( *++argv );
-					fprintf( stderr, "minimumweight = %f\n", minimumweight );
+//					fprintf( stderr, "minimumweight = %f\n", minimumweight );
 					--argc;
 					goto nextoption;
 #if 0
@@ -396,6 +463,17 @@ void arguments( int argc, char *argv[] )
 				case 'p':
 					smoothing = 1;
 					break;
+				case '=':
+					specifictarget = 1;
+					break;
+				case ':':
+					nwildcard = 1;
+					break;
+				case '+':
+					outputhat23 = myatoi( *++argv );
+//					fprintf( stderr, "outputhat23 = %f\n", outputhat23 );
+					--argc;
+					goto nextoption;
                 default:
                     fprintf( stderr, "illegal option %c\n", c );
                     argc = 0;
@@ -405,6 +483,9 @@ void arguments( int argc, char *argv[] )
 		nextoption:
 			;
 	}
+
+//	reporterr( "argc=%d\n", argc );
+
     if( argc == 1 )
     {
         cut = atof( (*argv) );
@@ -412,7 +493,7 @@ void arguments( int argc, char *argv[] )
     }
     if( argc != 0 ) 
     {
-        fprintf( stderr, "options: Check source file !\n" );
+        fprintf( stderr, "tbfast options: Check source file !\n" );
         exit( 1 );
     }
 	if( tbitr == 1 && outgap == 0 )
@@ -535,6 +616,7 @@ static void *treebasethread( void *arg )
 	RNApair ***singlerna = targ->singlerna;
 	double *effarr_kozo = targ->effarr_kozo;
 	int *fftlog = targ->fftlog;
+	int *targetmap = targ->targetmap;
 	char *mergeoralign = targ->mergeoralign;
 
 	char **mseq1, **mseq2;
@@ -549,6 +631,7 @@ static void *treebasethread( void *arg )
 	double *effarr1_kozo = NULL;
 	double *effarr2_kozo = NULL;
 	LocalHom ***localhomshrink = NULL;
+	char *swaplist = NULL;
 	int m1, m2;
 //	double dumfl = 0.0;
 	double dumdb = 0.0;
@@ -578,8 +661,10 @@ static void *treebasethread( void *arg )
 	indication2 = AllocateCharVec( 150 );
 #if 0
 #else
+	swaplist = NULL;
 	if( constraint )
 	{
+		if( specifictarget ) swaplist = calloc( njob, sizeof( char ) );
 		localhomshrink = (LocalHom ***)calloc( njob, sizeof( LocalHom ** ) );
 		for( i=0; i<njob; i++ )
 			localhomshrink[i] = (LocalHom **)calloc( njob, sizeof( LocalHom *) );
@@ -621,8 +706,10 @@ static void *treebasethread( void *arg )
 			commonIP = NULL;
 			Falign( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
 			Falign_udpari_long( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL );
-			A__align( NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
+			A__align( NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0, -1, -1 );
 			D__align( NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
+			partA__align( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL );
+			G__align11( NULL, NULL, NULL, 0, 0, 0 ); // iru?
 			free( mseq1 );
 			free( mseq2 );
 			free( localcopy );
@@ -651,6 +738,7 @@ static void *treebasethread( void *arg )
 					free( localhomshrink );
 					localhomshrink = NULL;
 				}
+				if( specifictarget ) free( swaplist );
 			}
 			return( NULL );
 		}
@@ -728,14 +816,19 @@ static void *treebasethread( void *arg )
 			clus1 = fastconjuction_noname_kozo( topol[l][0], localcopy, mseq1, effarr1, effarr, effarr1_kozo, effarr_kozo, indication1 );
 			clus2 = fastconjuction_noname_kozo( topol[l][1], localcopy, mseq2, effarr2, effarr, effarr2_kozo, effarr_kozo, indication2 );
 		}
+#if 0
+		else if( specifictarget )
+		{
+			clus1 = fastconjuction_target( topol[l][0], localcopy, mseq1, effarr1, effarr, indication1, minimumweight, targetmap );
+			clus2 = fastconjuction_target( topol[l][1], localcopy, mseq2, effarr2, effarr, indication2, minimumweight, targetmap );
+		}
+#endif
 		else
 		{
 			clus1 = fastconjuction_noname( topol[l][0], localcopy, mseq1, effarr1, effarr, indication1, minimumweight );
 			clus2 = fastconjuction_noname( topol[l][1], localcopy, mseq2, effarr2, effarr, indication2, minimumweight );
 		}
 
-
-
 #if 1
 		fprintf( stderr, "\rSTEP % 5d /%d (thread %4d) ", l+1, njob-1, thread_no );
 #else
@@ -762,7 +855,10 @@ static void *treebasethread( void *arg )
 
 		if( constraint )
 		{
-			fastshrinklocalhom( topol[l][0], topol[l][1], localhomtable, localhomshrink );
+			if( specifictarget )
+				fastshrinklocalhom_target( topol[l][0], topol[l][1], localhomtable, localhomshrink, swaplist, targetmap );
+			else
+				fastshrinklocalhom_half( topol[l][0], topol[l][1], localhomtable, localhomshrink );
 //			msfastshrinklocalhom( topol[l][0], topol[l][1], localhomtable, localhomshrink );
 //			fprintf( stderr, "localhomshrink =\n" );
 //			outlocalhompt( localhomshrink, clus1, clus2 );
@@ -816,13 +912,13 @@ static void *treebasethread( void *arg )
 			fprintf( stderr, "c" );
 			if( alg == 'A' )
 			{
-				imp_match_init_strict( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+				imp_match_init_strict( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, topol[l][0], topol[l][1] );
 				if( rnakozo ) imp_rna( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, NULL, NULL, NULL );
-				pscore = A__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
+				pscore = A__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap, -1, -1 );
 			}
 			if( alg == 'd' )
 			{
-				imp_match_init_strictD( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+				imp_match_init_strictD( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, topol[l][0], topol[l][1]  );
 				if( rnakozo ) imp_rnaD( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, NULL, NULL, NULL );
 				pscore = D__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 			}
@@ -857,7 +953,7 @@ static void *treebasethread( void *arg )
 					pscore = MSalignmm( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					break;
 				case( 'A' ):
-					pscore = A__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
+					pscore = A__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap, -1, -1 );
 					break;
 				case( 'd' ):
 					pscore = D__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
@@ -886,18 +982,19 @@ static void *treebasethread( void *arg )
 
 		if( disp ) display( localcopy, njob );
 
+
+
+
 		pthread_mutex_lock( targ->mutex );
 		dep[l].done = 1;
 		(*nrunpt)--;
 		pthread_cond_broadcast( targ->treecond );
 
-//		pthread_mutex_unlock( targ->mutex );
-//		pthread_mutex_lock( targ->mutex );
-
 		for( i=0; (j=topol[l][0][i])!=-1; i++ )
 			strcpy( aseq[j], localcopy[j] );
 		for( i=0; (j=topol[l][1][i])!=-1; i++ )
 			strcpy( aseq[j], localcopy[j] );
+
 		pthread_mutex_unlock( targ->mutex );
 
 		for( i=0; (j=topol[l][0][i])!=-1; i++ )
@@ -912,7 +1009,7 @@ static void *treebasethread( void *arg )
 }
 #endif
 
-void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq1, char **mseq2, int ***topol, Treedep *dep, double *effarr, int *alloclen, LocalHom **localhomtable, RNApair ***singlerna, double *effarr_kozo )
+void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq1, char **mseq2, int ***topol, Treedep *dep, double *effarr, int *alloclen, LocalHom **localhomtable, RNApair ***singlerna, double *effarr_kozo, int *targetmap, int *targetmapr, int ntarget )
 {
 	int i, l, m;
 	int len1nocommongap, len2nocommongap;
@@ -925,6 +1022,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 	double *effarr1_kozo = NULL;
 	double *effarr2_kozo = NULL;
 	LocalHom ***localhomshrink = NULL;
+	char *swaplist = NULL;
 	int *fftlog;
 	int m1, m2;
 	int *gaplen;
@@ -958,8 +1056,10 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 	dynamicmtx = AllocateDoubleMtx( nalphabets, nalphabets );
 #if 0
 #else
+	swaplist = NULL;
 	if( constraint )
 	{
+		if( specifictarget ) swaplist = calloc( njob, sizeof( char ) );
 		localhomshrink = (LocalHom ***)calloc( njob, sizeof( LocalHom ** ) );
 		for( i=0; i<njob; i++ )
 			localhomshrink[i] = (LocalHom **)calloc( njob, sizeof( LocalHom *) );
@@ -985,9 +1085,18 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 #endif
 
 
+
 	if( constraint )
-		calcimportance( njob, effarr, aseq, localhomtable );
+	{
+//		calcimportance( njob, effarr, aseq, localhomtable );
 //		dontcalcimportance( njob, effarr, aseq, localhomtable ); // CHUUIII!!!!!
+		if( specifictarget )
+			calcimportance_target( njob, ntarget, effarr, aseq, localhomtable, targetmap, targetmapr );
+//			dontcalcimportance_target( njob, effarr, aseq, localhomtable, ntarget ); // CHUUIII!!!!!
+		else
+//			calcimportance( njob, effarr, aseq, localhomtable );
+			calcimportance_half( njob, effarr, aseq, localhomtable );
+	}
 
 
 //	writePre( njob, name, nlen, aseq, 0 );
@@ -1036,6 +1145,13 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 			clus1 = fastconjuction_noname_kozo( topol[l][0], aseq, mseq1, effarr1, effarr, effarr1_kozo, effarr_kozo, indication1 );
 			clus2 = fastconjuction_noname_kozo( topol[l][1], aseq, mseq2, effarr2, effarr, effarr2_kozo, effarr_kozo, indication2 );
 		}
+#if 0
+		else if( specifictarget )
+		{
+			clus1 = fastconjuction_target( topol[l][0], aseq, mseq1, effarr1, effarr, indication1, minimumweight, targetmap );
+			clus2 = fastconjuction_target( topol[l][1], aseq, mseq2, effarr2, effarr, indication2, minimumweight, targetmap );
+		}
+#endif
 		else
 		{
 			clus1 = fastconjuction_noname( topol[l][0], aseq, mseq1, effarr1, effarr, indication1, minimumweight );
@@ -1090,7 +1206,10 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 
 		if( constraint )
 		{
-			fastshrinklocalhom( topol[l][0], topol[l][1], localhomtable, localhomshrink );
+			if( specifictarget )
+				fastshrinklocalhom_target( topol[l][0], topol[l][1], localhomtable, localhomshrink, swaplist, targetmap );
+			else
+				fastshrinklocalhom_half( topol[l][0], topol[l][1], localhomtable, localhomshrink );
 //			msfastshrinklocalhom( topol[l][0], topol[l][1], localhomtable, localhomshrink );
 //			fprintf( stderr, "localhomshrink =\n" );
 //			outlocalhompt( localhomshrink, clus1, clus2 );
@@ -1105,6 +1224,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 		}
 
 
+
 /*
 		fprintf( stderr, "before align all\n" );
 		display( aseq, njob );
@@ -1144,13 +1264,13 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 			fprintf( stderr, "c" );
 			if( alg == 'A' )
 			{
-				imp_match_init_strict( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+				imp_match_init_strict( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, topol[l][0], topol[l][1]  );
 				if( rnakozo ) imp_rna( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, NULL, NULL, NULL );
-				pscore = A__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
+				pscore = A__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap, topol[l][0][0], 1 ); // reuse profiles
 			}
 			if( alg == 'd' )
 			{
-				imp_match_init_strictD( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+				imp_match_init_strictD( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, topol[l][0], topol[l][1]  );
 				if( rnakozo ) imp_rnaD( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, NULL, NULL, NULL );
 				pscore = D__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 			}
@@ -1185,7 +1305,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 					pscore = MSalignmm( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					break;
 				case( 'A' ):
-					pscore = A__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
+					pscore = A__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap, topol[l][0][0], 1 ); // reuse profiles
 					break;
 				case( 'd' ):
 					pscore = D__align( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
@@ -1275,6 +1395,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 			free( localhomshrink );
 			localhomshrink = NULL;
 		}
+		if( specifictarget ) free( swaplist );
 	}
 
 	free( topol );
@@ -1289,11 +1410,11 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 	FreeDoubleMtx( dynamicmtx );
 	free( effarr1_kozo );
 	free( effarr2_kozo );
+	Falign( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
 	D__align( NULL,  NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
-	A__align( NULL,  NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
-	imp_match_init_strictD( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0 );
-	imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0 );
-	freeconstants();
+	A__align( NULL,  NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0, -1, -1 );
+	imp_match_init_strictD( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, NULL );
+	imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, NULL );
 	FreeCommonIP();
 }
 
@@ -1400,15 +1521,21 @@ int main( int argc, char *argv[] )
 	char c;
 	int alloclen;
 	LocalHom **localhomtable = NULL;
+	LocalHom *tmpptr;
 	RNApair ***singlerna = NULL;
 	double ssi, ssj, bunbo;
 	static char *kozoarivec;
 	int nkozo;
+	int ntarget;
+	int *targetmap, *targetmapr;
+	int ilim, jst, jj;
+	int pac, tac;
+	char **pav, **tav;
 
-	arguments( argc, argv );
-#ifndef enablemultithread
-	nthread = 0;
-#endif
+	pav = calloc( argc, sizeof( char * ) );
+	tav = calloc( argc, sizeof( char * ) );
+
+	arguments( argc, argv, &pac, pav, &tac, tav );
 
 	if( fastathreshold < 0.0001 ) constraint = 0; 
 
@@ -1428,6 +1555,7 @@ int main( int argc, char *argv[] )
 	rewind( infp );
 
 
+
 	nkozo = 0;
 
 	if( njob < 2 )
@@ -1472,13 +1600,53 @@ int main( int argc, char *argv[] )
 
 	ndeleted = 0;
 
-	if( constraint )
+#if 0
+	readData( infp, name, nlen, seq );
+#else
+	readData_pointer( infp, name, nlen, seq );
+	fclose( infp );
+#endif
+
+	if( specifictarget )
 	{
-		localhomtable = (LocalHom **)calloc( njob, sizeof( LocalHom *) );
+		targetmap = calloc( njob, sizeof( int ) );
+		ntarget = 0;
 		for( i=0; i<njob; i++ )
 		{
-			localhomtable[i] = (LocalHom *)calloc( njob, sizeof( LocalHom ) );
-			for( j=0; j<njob; j++ )
+			targetmap[i] = -1;
+			if( !strncmp( name[i]+1, "_focus_", 7 ) )
+				targetmap[i] = ntarget++;
+		}
+		targetmapr = calloc( ntarget, sizeof( int ) );
+		for( i=0; i<njob; i++ )
+			if( targetmap[i] != -1 ) targetmapr[targetmap[i]] = i;
+
+	}
+	else
+	{
+		ntarget = njob;
+		targetmap = calloc( njob, sizeof( int ) );
+		targetmapr = calloc( njob, sizeof( int ) );
+		for( i=0; i<njob; i++ )
+			targetmap[i] = targetmapr[i] = i;
+	}
+
+#if 0
+	for( i=0; i<njob; i++ )
+		reporterr( "targetmap[%d] = %d\n", i, targetmap[i] );
+	for( i=0; i<ntarget; i++ )
+		reporterr( "targetmapr[%d] = %d\n", i, targetmapr[i] );
+#endif
+
+	if( constraint && !noalign ) // 2016mar15 noalign tsuika
+	{
+
+		ilim = njob;
+		localhomtable = (LocalHom **)calloc( ntarget, sizeof( LocalHom *) );
+		for( i=0; i<ntarget; i++ )
+		{
+			localhomtable[i] = (LocalHom *)calloc( ilim, sizeof( LocalHom ) );
+			for( j=0; j<ilim; j++ )
 			{
 				localhomtable[i][j].start1 = -1;
 				localhomtable[i][j].end1 = -1;
@@ -1488,16 +1656,116 @@ int main( int argc, char *argv[] )
 				localhomtable[i][j].opt = -1.0;
 				localhomtable[i][j].importance = -1.0;
 				localhomtable[i][j].next = NULL;
+				localhomtable[i][j].nokori = 0;
+				localhomtable[i][j].extended = -1;
+				localhomtable[i][j].last = localhomtable[i]+j;
 				localhomtable[i][j].korh = 'h';
 			}
+			if( !specifictarget ) ilim--;
 		}
 
-		fprintf( stderr, "Loading 'hat3' ... " );
-		prep = fopen( "hat3", "r" );
-		if( prep == NULL ) ErrorExit( "Make hat3." );
-		readlocalhomtable( prep, njob, localhomtable, kozoarivec );
-		fclose( prep );
-		fprintf( stderr, "\ndone.\n" );
+//		reporterr( "pac=%d\n", pac );
+//		reporterr( "pav[0]=%s\n", pav[0] );
+		if( callpairlocalalign )
+		{
+			pairlocalalign( njob, nlenmax, name, seq, iscore, localhomtable, pac, pav );
+			arguments( tac, tav, NULL, NULL, NULL, NULL ); // anzen no tame
+			callpairlocalalign = 1; // wakarinikui.
+			if( fastathreshold < 0.0001 ) constraint = 0; 
+//			fprintf( stderr, "blosum %d / kimura 200\n", nblosum );
+//			fprintf( stderr, "scoremtx=%d\n", scoremtx );
+//			fprintf( stderr, "fastathreshold=%f\n", fastathreshold );
+//			fprintf( stderr, "constraing=%d\n", constraint );
+//exit( 1 );
+			for( ilim=njob, i=0; i<ntarget; i++ )
+			{
+				for( j=0; j<ilim; j++ )
+				{
+					for( tmpptr=localhomtable[i]+j; tmpptr; tmpptr=tmpptr->next )
+					{
+						if( tmpptr->opt == -1.0 ) continue;
+#if SHISHAGONYU // for debug
+						char buff[100];
+						sprintf( buff, "%10.5f", tmpptr->opt );
+						tmpptr->opt = 0.0;
+						sscanf( buff, "%lf", &(tmpptr->opt) );
+#endif
+						tmpptr->opt = ( tmpptr->opt ) / 5.8  * 600;
+					}
+				}
+				if( !specifictarget ) ilim--;
+			}
+
+			prep = fopen( "hat3.seed", "r" );
+			if( prep )
+			{
+				fprintf( stderr, "Loading 'hat3.seed' ... " );
+				if( specifictarget ) readlocalhomtable2_target( prep, njob, localhomtable, kozoarivec, targetmap ); // uwagakisarerukara koredehadame.
+				else readlocalhomtable2_half( prep, njob, localhomtable, kozoarivec );                                       // uwagakisarerukara koredehadame.
+				fclose( prep );
+				fprintf( stderr, "\ndone.\n" );
+			}
+			else
+				fprintf( stderr, "No hat3.seed.\n" );
+
+			if( outputhat23 )
+			{
+				prep = fopen( "hat3", "w" );
+				if( !prep ) ErrorExit( "Cannot open hat3 to write." );
+
+				fprintf( stderr, "Writing hat3 for iterative refinement\n" );
+				if( specifictarget )
+					ilim = ntarget;
+				else
+					ilim = njob-1;	
+				for( i=0; i<ilim; i++ ) 
+				{
+					if( specifictarget )
+					{
+						jst = 0;
+						jj = 0;
+					}
+					else
+					{
+						jst = i;
+						jj = 0;
+					}
+					for( j=jst; j<njob; j++, jj++ )
+					{
+						for( tmpptr=localhomtable[i]+jj; tmpptr; tmpptr=tmpptr->next )
+						{
+							if( tmpptr->opt == -1.0 ) continue;
+							if( targetmap[j] == -1 || targetmap[i] < targetmap[j] )
+								fprintf( prep, "%d %d %d %7.5f %d %d %d %d %c\n", targetmapr[i], j, tmpptr->overlapaa, tmpptr->opt/600*5.8, tmpptr->start1, tmpptr->end1, tmpptr->start2, tmpptr->end2, tmpptr->korh );
+						}
+					}
+				}
+				fclose( prep );
+
+				prep = fopen( "hat2", "w" );
+				WriteFloatHat2_pointer_halfmtx( prep, njob, name, iscore );
+				fclose( prep );
+			}
+			else if( distout ) // choufuku shiterukedo, muda deha nai.
+			{
+				prep = fopen( "hat2", "w" );
+				WriteFloatHat2_pointer_halfmtx( prep, njob, name, iscore );
+				fclose( prep );
+			}
+		}
+		else
+		{
+			fprintf( stderr, "Loading 'hat3' ... " );
+			prep = fopen( "hat3", "r" );
+			if( prep == NULL ) ErrorExit( "Make hat3." );
+			if( specifictarget ) readlocalhomtable2_target( prep, njob, localhomtable, kozoarivec, targetmap );
+			else readlocalhomtable2_half( prep, njob, localhomtable, kozoarivec );
+			fclose( prep );
+			fprintf( stderr, "\ndone.\n" );
+		}
+		free( tav );
+		free( pav );
+
 
 
 		nkozo = 0;
@@ -1516,7 +1784,13 @@ int main( int argc, char *argv[] )
 		}
 
 
-//		outlocalhom( localhomtable, njob );
+#if 0
+		if( specifictarget )
+			outlocalhom_target( localhomtable, ntarget, njob );
+		else
+			outlocalhom_half( localhomtable, njob );
+		exit( 1 );
+#endif
 
 #if 0
 		fprintf( stderr, "Extending localhom ... " );
@@ -1524,13 +1798,31 @@ int main( int argc, char *argv[] )
 		fprintf( stderr, "done.\n" );
 #endif
 	}
+	else
+	{
+
+		if( callpairlocalalign )
+		{
+			pairlocalalign( njob, nlenmax, name, seq, iscore, NULL, pac, pav );
+			arguments( tac, tav, NULL, NULL, NULL, NULL ); // anzen no tame
+			free( tav );
+			free( pav );
+			callpairlocalalign = 1; // wakarinikui.
+			if( fastathreshold < 0.0001 ) constraint = 0; 
+			fprintf( stderr, "blosum %d / kimura 200\n", nblosum );
+			fprintf( stderr, "scoremtx=%d\n", scoremtx );
+			fprintf( stderr, "fastathreshold=%f\n", fastathreshold );
+		}
+		if( distout || outputhat23 )
+		{
+			prep = fopen( "hat2", "w" );
+			WriteFloatHat2_pointer_halfmtx( prep, njob, name, iscore );
+			fclose( prep );
+		}
+	}
+
+
 
-#if 0
-	readData( infp, name, nlen, seq );
-#else
-	readData_pointer( infp, name, nlen, seq );
-	fclose( infp );
-#endif
 
 	constants( njob, seq );
 
@@ -1587,8 +1879,7 @@ int main( int argc, char *argv[] )
 			exit( 1 );
 		}
 #endif
-		fprintf( stderr, "Loading a tree ... " );
-		loadtree( njob, topol, len, name, nlen, dep );
+		loadtree( njob, topol, len, name, nlen, dep, treeout );
 //		loadtop( njob, topol, len, name, NULL, dep ); // 2015/Jan/13, not yet checked
 		fprintf( stderr, "\ndone.\n\n" );
 	}
@@ -1596,6 +1887,10 @@ int main( int argc, char *argv[] )
 	{
 		if( tbutree == 0 )
 		{
+			reporterr( "Bug.  This function should not be used in versions >7.2.  Please email kazutaka.katoh@aist.go.jp\n" );
+			fflush( stderr );
+			exit( 1 );
+
 			for( i=1; i<njob; i++ ) 
 			{
 				if( nlen[i] != nlen[0] ) 
@@ -1605,8 +1900,6 @@ int main( int argc, char *argv[] )
 				}
 			}
 	
-			fprintf( stderr, "Making a distance matrix .. (Should not be used in versions >7.2) \n" );
-			fflush( stderr );
 			skiptable = AllocateIntMtx( njob, 0 );
 			makeskiptable( njob, skiptable, seq ); // allocate suru.
 			ien = njob-1;
@@ -1687,46 +1980,64 @@ int main( int argc, char *argv[] )
 					}
 				}
 			}
-			fprintf( stderr, "\ndone.\n\n" );
+//			fprintf( stderr, "\ndone.\n\n" );
 			FreeIntMtx( skiptable );
-			fflush( stderr );
+//			fflush( stderr );
 		}
 		else
 		{
-			if( multidist )
+			if( callpairlocalalign )
 			{
-				fprintf( stderr, "Loading 'hat2n' (aligned sequences - new sequences) ... " );
-				prep = fopen( "hat2n", "r" );
-				if( prep == NULL ) ErrorExit( "Make hat2." );
-				readhat2_doublehalf_pointer( prep, njob, name, iscore );
+				if( multidist )
+				{
+					reporterr( "Bug in v7.290.  Please email kazutaka.katoh@aist.go.jp\n" );
+					exit( 1 );
+				}
+#if 0
+				prep = fopen( "hat2", "w" );
+				if( !prep ) ErrorExit( "Cannot open hat2." );
+				WriteFloatHat2_pointer_halfmtx( prep, njob, name, iscore ); // jissiha double
 				fclose( prep );
-				fprintf( stderr, "done.\n" );
-			
-				fprintf( stderr, "Loading 'hat2i' (aligned sequences) ... " );
-				prep = fopen( "hat2i", "r" );
-				if( prep == NULL ) ErrorExit( "Make hat2i." );
-				readhat2_doublehalf_pointer( prep, njob-nadd, name, iscore );
-				fclose( prep );
-				fprintf( stderr, "done.\n" );
+#endif
 			}
 			else
 			{
-				fprintf( stderr, "Loading 'hat2' ... " );
-				prep = fopen( "hat2", "r" );
-				if( prep == NULL ) ErrorExit( "Make hat2." );
-				readhat2_doublehalf_pointer( prep, njob, name, iscore );
-				fclose( prep );
-				fprintf( stderr, "done.\n" );
+				if( multidist )
+				{
+					fprintf( stderr, "Loading 'hat2n' (aligned sequences - new sequences) ... " );
+					prep = fopen( "hat2n", "r" );
+					if( prep == NULL ) ErrorExit( "Make hat2." );
+					readhat2_doublehalf_pointer( prep, njob, name, iscore );
+					fclose( prep );
+					fprintf( stderr, "done.\n" );
+				
+					fprintf( stderr, "Loading 'hat2i' (aligned sequences) ... " );
+					prep = fopen( "hat2i", "r" );
+					if( prep == NULL ) ErrorExit( "Make hat2i." );
+					readhat2_doublehalf_pointer( prep, njob-nadd, name, iscore );
+					fclose( prep );
+					fprintf( stderr, "done.\n" );
+				}
+				else
+				{
+					fprintf( stderr, "Loading 'hat2' ... " );
+					prep = fopen( "hat2", "r" );
+					if( prep == NULL ) ErrorExit( "Make hat2." );
+					readhat2_doublehalf_pointer( prep, njob, name, iscore );
+					fclose( prep );
+					fprintf( stderr, "done.\n" );
+				}
+
+				if( distout ) // callpairlocalalign == 1 no toki ha ue de shorizumi.
+				{
+					hat2p = fopen( "hat2", "w" );
+					WriteFloatHat2_pointer_halfmtx( hat2p, njob, name, iscore );
+					fclose( hat2p );
+				}
 			}
+//			for( i=0; i<njob-1; i++ ) for( j=i+1; j<njob; j++ ) printf( "dist %d-%d = %f\n", i, j, iscore[i][j-i] );
 		}
-#if 1
-		if( distout )
-		{
-			hat2p = fopen( "hat2", "w" );
-			WriteFloatHat2_pointer_halfmtx( hat2p, njob, name, iscore );
-			fclose( hat2p );
-		}
-#endif
+
 		if( nkozo )
 		{
 			ien = njob-1;
@@ -1783,6 +2094,7 @@ int main( int argc, char *argv[] )
 	}
 
 
+
 	orderfp = fopen( "order", "w" );
 	if( !orderfp )
 	{
@@ -2168,7 +2480,13 @@ int main( int argc, char *argv[] )
 		for( i=0; i<njob; i++ ) fftlog[i] = 1;
 
 		if( constraint )
-			calcimportance( njob, eff, bseq, localhomtable );
+		{
+			if( specifictarget )
+				calcimportance_target( njob, ntarget, eff, bseq, localhomtable, targetmap, targetmapr );
+//				dontcalcimportance_target( njob, eff, bseq, localhomtable, ntarget ); // CHUUIII!!!!!
+			else
+				calcimportance_half( njob, eff, bseq, localhomtable );
+		}
 //			dontcalcimportance( njob, eff, bseq, localhomtable ); // CHUUUUIIII!!!
 
 		for( i=0; i<nthread_yoyu; i++ )
@@ -2188,6 +2506,7 @@ int main( int argc, char *argv[] )
 			targ[i].effarr_kozo = eff_kozo_mapped;
 			targ[i].fftlog = fftlog;
 			targ[i].mergeoralign = mergeoralign;
+			targ[i].targetmap = targetmap;
 			targ[i].mutex = &mutex;
 			targ[i].treecond = &treecond;
 
@@ -2203,11 +2522,15 @@ int main( int argc, char *argv[] )
 		free( handle );
 		free( targ );
 		free( fftlog );
+//		free( topol[njob-1][0] );
+//		free( topol[njob-1][1] );
+		free( topol[njob-1] );
+		free( topol );
 	}
 	else
 #endif
 
-		treebase( nlen, bseq, nadd, mergeoralign, mseq1, mseq2, topol, dep, eff, &alloclen, localhomtable, singlerna, eff_kozo_mapped );
+		treebase( nlen, bseq, nadd, mergeoralign, mseq1, mseq2, topol, dep, eff, &alloclen, localhomtable, singlerna, eff_kozo_mapped, targetmap, targetmapr, ntarget );
 	fprintf( stderr, "\ndone.\n" );
 
 	
@@ -2282,6 +2605,7 @@ int main( int argc, char *argv[] )
 	fprintf( trap_g, "done.\n" );
 //	fclose( trap_g );
 	free( mergeoralign );
+	freeconstants();
 
 
 
@@ -2312,8 +2636,15 @@ int main( int argc, char *argv[] )
 	fprintf( stderr, "OSHIMAI\n" );
 #endif
 
-	if( constraint ) FreeLocalHomTable( localhomtable, njob );
-
+	if( constraint ) 
+	{
+		if( specifictarget )
+			FreeLocalHomTable_part( localhomtable, ntarget, njob );
+		else
+			FreeLocalHomTable_half( localhomtable, njob );
+	}
+	free( targetmap );
+	free( targetmapr );
 
 
 	if( spscoreout ) reporterr( "Unweighted sum-of-pairs score = %10.5f\n", sumofpairsscore( njob, bseq ) );
@@ -2350,6 +2681,7 @@ int main( int argc, char *argv[] )
 	free( eff );
 	free( dep );
 	closeFiles();
+	if( nadd ) free( addmem );
 
 	return( 0 );
 }
