@@ -157,6 +157,29 @@ void cpmx_calc_new_bk( char **seq, double **cpmx, double *eff, int lgth, int clu
     }
 }
 
+void cpmx_calc_add( char **seq, double **cpmx, double *eff, int lgth, int clus ) // lastmem = newmem; summ eff must be 1.0
+{
+	double neweff, orieff;
+	int newmem, i, j;
+
+	newmem = clus-1;
+	neweff = eff[clus-1];
+	orieff = 1.0-neweff;
+#if 1 // TESTING Feb/1/18:00
+	for( j=0; j<lgth; j++ )
+	{
+		for( i=0;i<nalphabets; i++ ) cpmx[i][j] *= orieff;
+		cpmx[(unsigned char)amino_n[(unsigned char)seq[newmem][j]]][j] += neweff;
+	}
+#else // possibly faster?
+	for( i=0;i<nalphabets; i++ ) 
+	{
+		for( j=0; j<lgth; j++ ) cpmx[i][j] *= orieff;
+	}
+	for( j=0; j<lgth; j++ ) cpmx[(unsigned char)amino_n[(unsigned char)seq[newmem][j]]][j] += neweff;
+#endif
+}
+
 void cpmx_calc_new( char **seq, double **cpmx, double *eff, int lgth, int clus ) // summ eff must be 1.0
 {
 	int  i, j, k;
@@ -449,6 +472,51 @@ int fastconjuction_noname_kozo( int *memlist, char **seq, char **aseq, double *p
 }
 
 
+#if 0
+int fastconjuction_target( int *memlist, char **seq, char **aseq, double *peff, double *eff, char *d, double mineff, int *targetmap )
+{
+	int m, k, dln;
+	char b[B];
+	double total;
+	int *memlistbk = memlist;
+
+#if DEBUG
+	fprintf( stderr, "s = %d\n", s );
+#endif
+
+	total = 0.0;
+	d[0] = 0;
+	dln = 0;
+	for( k=0; *memlist!=-1; memlist++, k++ )
+	{
+		m = *memlist;
+		dln += sprintf( b, " %d", m+1 ); 
+#if 1
+		if( dln < 100 ) strcat( d, b );
+#else
+		strcat( d, b );
+#endif
+		aseq[k] = seq[m];
+		if( eff[m] < mineff )
+			peff[k] = mineff;
+		else
+			peff[k] = eff[m];
+
+		total += peff[k];
+	}
+#if 1
+	for( m=0; m<k; m++ )
+	{
+//		fprintf( stderr, "Apr17   peff[%d] = %f\n", m, peff[m] );
+		peff[m] /= total;
+	}
+#endif
+
+	return( k );
+}
+#endif
+
+
 int fastconjuction_noname( int *memlist, char **seq, char **aseq, double *peff, double *eff, char *d, double mineff  )
 {
 	int m, k, dln;
@@ -839,6 +907,77 @@ int shrinklocalhom( char **pair, int s1, int s2, LocalHom **localhom, LocalHom *
 	}
 	return( 0 );
 }
+int msshrinklocalhom_fast_target( int *memlist1, int *memlist2, LocalHom **localhom, LocalHom ***localhomshrink, char *swaplist, int *targetmap )
+{
+	int m1, k1, m2, k2, t1, i2;
+
+	for( k1=0; (m1=memlist1[k1])!=-1; k1++ )
+	{
+		if( targetmap[m1] == -1 )
+		{	
+			swaplist[k1] = 1;
+//			swaplist[k1] = 0; // DAME!!!
+			for( k2=0; (m2=memlist2[k2])!=-1; k2++ )
+			{
+				if( targetmap[m2] == -1 )
+				{
+					localhomshrink[k1][k2] = NULL;
+					continue;
+				}
+		
+				t1 = targetmap[m2]; // start1 <-> start2, end1 <-> end2
+				i2 = m1;
+
+				if( localhom[t1][i2].opt == -1 )
+					localhomshrink[k1][k2] = NULL;
+				else
+					localhomshrink[k1][k2] = localhom[t1]+i2;
+			}
+		}
+		else
+		{
+			swaplist[k1] = 0;
+			for( k2=0; (m2=memlist2[k2])!=-1; k2++ )
+			{
+				t1 = targetmap[m1];
+				i2 = m2;
+	
+				if( localhom[t1][i2].opt == -1 )
+					localhomshrink[k1][k2] = NULL;
+				else
+					localhomshrink[k1][k2] = localhom[t1]+i2;
+			}
+		}
+	}
+	return( 0 );
+}
+
+int msshrinklocalhom_fast_half( int *memlist1, int *memlist2, LocalHom **localhom, LocalHom ***localhomshrink )
+{
+	int m1, k1, m2, k2;
+
+	for( k1=0; (m1=memlist1[k1])!=-1; k1++ )
+	{
+		for( k2=0; (m2=memlist2[k2])!=-1; k2++ )
+		{
+			if( m1 < m2 )
+			{
+				if( localhom[m1][m2-m1].opt == -1 )
+					localhomshrink[k1][k2] = NULL;
+				else
+					localhomshrink[k1][k2] = localhom[m1]+m2-m1;
+			}
+			else
+			{
+				if( localhom[m2][m1-m2].opt == -1 )
+					localhomshrink[k1][k2] = NULL;
+				else
+					localhomshrink[k1][k2] = localhom[m2]+m1-m2;
+			}
+		}
+	}
+	return( 0 );
+}
 
 int msshrinklocalhom_fast( int *memlist1, int *memlist2, LocalHom **localhom, LocalHom ***localhomshrink )
 {
@@ -894,6 +1033,101 @@ int fastshrinklocalhom( int *mem1, int *mem2, LocalHom **localhom, LocalHom ***l
 				localhomshrink[k1][k2] = NULL;
 			else
 				localhomshrink[k1][k2] = localhom[*intpt1]+*intpt2;
+
+//			if( localhomshrink[k1][k2] != NULL )
+//				printf( "ori localhomshrink[%d][%d].opt = %f\n", k1, k2, localhomshrink[k1][k2]->opt );
+		}
+	}
+	return( 0 );
+}
+
+
+int fastshrinklocalhom_half( int *mem1, int *mem2, LocalHom **localhom, LocalHom ***localhomshrink )
+{
+	int k1, k2;
+	int *intpt1, *intpt2;
+
+	
+	for( intpt1=mem1, k1=0; *intpt1!=-1; intpt1++, k1++ )
+	{
+		for( intpt2=mem2, k2=0; *intpt2!=-1; intpt2++, k2++ )
+		{
+			if( *intpt1 < *intpt2 )
+			{
+				if( localhom[*intpt1][*intpt2-*intpt1].opt == -1 )
+					localhomshrink[k1][k2] = NULL;
+				else
+					localhomshrink[k1][k2] = localhom[*intpt1]+*intpt2-*intpt1;
+			}
+			else
+			{
+				if( localhom[*intpt2][*intpt1-*intpt2].opt == -1 )
+					localhomshrink[k1][k2] = NULL;
+				else
+					localhomshrink[k1][k2] = localhom[*intpt2]+*intpt1-*intpt2;
+			}
+
+//			if( localhomshrink[k1][k2] != NULL )
+//				printf( "ori localhomshrink[%d][%d].opt = %f\n", k1, k2, localhomshrink[k1][k2]->opt );
+		}
+	}
+	return( 0 );
+}
+
+
+int fastshrinklocalhom_target( int *mem1, int *mem2, LocalHom **localhom, LocalHom ***localhomshrink, char *swaplist, int *targetmap )
+{
+	int k1, k2;
+	int *intpt1, *intpt2;
+	int t1, i2;
+
+	
+	for( intpt1=mem1, k1=0; *intpt1!=-1; intpt1++, k1++ )
+	{
+		if( targetmap[*intpt1] == -1 )
+		{
+			swaplist[k1] = 1;
+//			swaplist[k1] = 0; // DAME!!!
+			for( intpt2=mem2, k2=0; *intpt2!=-1; intpt2++, k2++ )
+			{
+				if( targetmap[*intpt2] == -1 )
+				{
+					localhomshrink[k1][k2] = NULL;
+					continue;
+				}
+	
+				t1 = targetmap[*intpt2]; // end1<->end2, start1<->start2
+				i2 = *intpt1;
+	
+				if( localhom[t1][i2].opt == -1 )
+					localhomshrink[k1][k2] = NULL;
+				else
+					localhomshrink[k1][k2] = localhom[t1]+i2;
+	
+//				if( localhomshrink[k1][k2] != NULL )
+//					printf( "localhomshrink[%d][%d].opt = %f\n", k1, k2, localhomshrink[k1][k2]->opt );
+//				else
+//					printf( "localhomshrink[%d][%d] = NULL\n", k1, k2 );
+			}
+		}
+		else
+		{
+			swaplist[k1] = 0;
+			for( intpt2=mem2, k2=0; *intpt2!=-1; intpt2++, k2++ )
+			{
+				t1 = targetmap[*intpt1];
+				i2 = *intpt2;
+	
+				if( localhom[t1][i2].opt == -1 )
+					localhomshrink[k1][k2] = NULL;
+				else
+					localhomshrink[k1][k2] = localhom[t1]+i2;
+	
+//				if( localhomshrink[k1][k2] != NULL )
+//					printf( "localhomshrink[%d][%d].opt = %f\n", k1, k2, localhomshrink[k1][k2]->opt );
+//				else
+//					printf( "localhomshrink[%d][%d] = NULL\n", k1, k2 );
+			}
 		}
 	}
 	return( 0 );
